@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@
 /* Some constants */
 #define UDM_LANGPERDOC				16		/* FIXME */
 #define UDM_USER_AGENT				"MnoGoSearch/" VERSION
-#define UDM_MAXWORDPERQUERY			32
+#define UDM_MAXWORDPERQUERY			64
 
 /* Some sizes and others definitions */
 #define UDM_MAXDOCSIZE				2*1024*1024	/**< 2 MB  */
@@ -121,7 +121,7 @@
 #define UDM_ERROR		1
 #define UDM_NOTARGET		2
 #define UDM_TERMINATED		3
-
+#define UDM_UNSUPPORTED         4
 
 /* Flags for indexing */
 #define UDM_FLAG_REINDEX           1
@@ -294,6 +294,36 @@ typedef uint4 udm_pos_t;
 typedef unsigned char udm_secno_t;
 typedef unsigned char udm_wordnum_t;
 
+/**************************/
+typedef struct
+{
+  urlid_t       url_id;
+  udm_pos_t     seclen;
+  udm_pos_t     pos;
+  udm_wordnum_t num;
+  udm_secno_t   secno;
+} UDM_URL_CRD;
+
+
+typedef struct
+{
+  size_t       acoords;
+  size_t       ncoords;
+  size_t       order;
+  char	       *word;
+  UDM_URL_CRD  *Coords;
+} UDM_URLCRDLIST;
+
+
+typedef struct
+{
+  int                freeme;
+  size_t             nlists;
+  UDM_URLCRDLIST     *List;
+} UDM_URLCRDLISTLIST;
+
+/***************************/
+
 typedef struct udm_coord2_st
 {
   udm_pos_t pos;
@@ -345,9 +375,36 @@ typedef struct {
   UDM_URL_SCORE	*Item;
 } UDM_URLSCORELIST;
 
+
+/* UserScore and UserSiteScore structure */
+
+typedef struct udm_url_int4_st
+{
+  urlid_t url_id;
+  int4  param;
+} UDM_URL_INT4;
+
+typedef struct udm_url_int4_list_st
+{
+  size_t nitems;
+  UDM_URL_INT4 *Item;
+} UDM_URL_INT4_LIST;
+
+
+
+/* Structure to handle limits */
+typedef struct udm_urlid_list_st
+{
+  char empty;
+  char exclude;
+  urlid_t *urls;
+  size_t nurls;
+} UDM_URLID_LIST;
+
+
 typedef struct {
         urlid_t         url_id;
-        uint4           coord;
+        uint4           score;
         uint4           per_site;
         urlid_t         site_id;
         time_t          last_mod_time;
@@ -422,6 +479,7 @@ typedef struct {
 	int	use_clones;	/**< Whether to detect clones */
         int	doc_per_site;
         int	collect_links;
+        int	crawl_delay;
 } UDM_SPIDERPARAM;
 
 /*****************************************************************/
@@ -519,7 +577,7 @@ typedef struct udm_conn_struct {
         int	status;
         int	connected;
         int	err;
-        int	retry;
+        time_t	host_last_used;
         int	conn_fd;
 #ifdef WIN32
         unsigned short port;
@@ -854,6 +912,7 @@ typedef struct udm_config_struct {
         UDM_MATCHLIST   SectionHdrMatch;/**< User sections after headers*/
         UDM_MATCHLIST   SectionGsrMatch;/**< User sections after quesser*/
         UDM_MATCHLIST   SectionMatch;   /**< User sections after parser */
+        UDM_MATCHLIST	Encodings;	/**< For AddType commands       */
 	
 	UDM_HREFLIST	Hrefs;		/**< Links cache                */
 	UDM_RESULT	Targets;	/**< Targets cache              */
