@@ -1,4 +1,4 @@
-/* Copyright (C) 2005 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -162,8 +162,14 @@ typedef uint4 udmcrc32_t;
 #ifdef USE_PARANOIA
 extern void *UdmViolationEnter(void);
 extern void UdmViolationExit(void *);
+#define UdmDebugEnter()            void *paran= UdmViolationEnter();
+#define UdmDebugReturn(rc)         {UdmViolationExit(paran);return rc;}
+#define UdmDebugReturnVoid()       {UdmViolationExit(paran);return;}
 #else
 #define UdmViolationExit(x)
+#define UdmDebugEnter()           /* UdmDebugEnter() */
+#define UdmDebugReturn(rc)        {return rc;}
+#define UdmDebugReturnVoid()      {return;}
 #endif
 
 #ifndef PATH_MAX
