@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,8 +26,8 @@
 #define UDM_ENCODING_UNKNOWN  7
 */
 
-extern __C_LINK int __UDMCALL UdmUnGzip(UDM_DOCUMENT *Doc);
-extern __C_LINK int __UDMCALL UdmInflate(UDM_DOCUMENT *Doc);
-extern __C_LINK int __UDMCALL UdmUncompress(UDM_DOCUMENT *Doc);
+extern __C_LINK int __UDMCALL UdmDocUnGzip(UDM_DOCUMENT *Doc);
+extern __C_LINK int __UDMCALL UdmDocInflate(UDM_DOCUMENT *Doc);
+extern __C_LINK int __UDMCALL UdmDocUncompress(UDM_DOCUMENT *Doc);
 
 #endif
