@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,14 +20,11 @@
 
 #include <limits.h>
 
-#if (HAVE_PGSQL)
-#include <libpq-fe.h>
-#endif
-
 #define UDM_SQLTYPE_UNKNOWN       0
 #define UDM_SQLTYPE_LONGVARBINARY 1
 #define UDM_SQLTYPE_LONGVARCHAR   2
 #define UDM_SQLTYPE_VARCHAR       3
+#define UDM_SQLTYPE_INT           4
 
 #define UDM_SQL_NULL_DATA	(-1)
 #define UDM_SQL_DATA_AT_EXEC	(-2)
@@ -65,14 +62,14 @@ typedef struct {
 	UDM_SQLFIELD	*Fields;
 	UDM_PSTR	*Items;
 	
-#ifdef HAVE_PGSQL
-	PGresult  *pgsqlres;
-#endif
 	void *specific;
 	struct udm_db_st *db;
 } UDM_SQLRES;
 
 
+int
+UdmSQLResFreeGeneric(UDM_SQLRES *res);
+
 typedef struct udm_sql_res_list_st
 {
   size_t nitems;
@@ -84,6 +81,8 @@ int UdmSQLResListAdd(UDM_SQLRESLIST *List, UDM_SQLRES *Res);
 void UdmSQLResListFree(UDM_SQLRESLIST *List);
 
 
+/********************************************/
+
 typedef struct {
 	char *word;
 	size_t nintags;
@@ -116,6 +115,8 @@ typedef struct {
 	urlid_t *urls;
 } UDM_MULTI_CACHE;
 
+/****************************************/
+
 typedef struct {
 	char *word;
 	urlid_t url_id;
@@ -134,6 +135,9 @@ typedef struct {
 	UDM_BLOB_CACHE_WORD *words;
 } UDM_BLOB_CACHE;
 
+
+/********************************************/
+
 typedef struct {
 	urlid_t url_id;
 	char    *word;
@@ -153,6 +157,8 @@ typedef struct {
 	urlid_t *urls;
 } UDM_WORD_CACHE;
 
+
+/*******************************************/
 struct udm_db_st;
 
 
@@ -160,12 +166,14 @@ typedef struct udm_db_handler_st
 {
   char	*(*SQLEscStr)(struct udm_db_st *db, char *to, const char *from, size_t len);
   int    (*SQLQuery)(struct udm_db_st *db, UDM_SQLRES *R, const char *query);
+  int    (*SQLConnect)(struct udm_db_st *db);
   void   (*SQLClose)(struct udm_db_st *db);
   int    (*SQLBegin)(struct udm_db_st *db);
   int    (*SQLCommit)(struct udm_db_st *db);
   int    (*SQLPrepare)(struct udm_db_st *db, const char *query);
   int    (*SQLBind)(struct udm_db_st *db, int position, void *data, int size, int type);
   int    (*SQLExec)(struct udm_db_st *db);
+  int    (*SQLStmtFree)(struct udm_db_st *db);
   int    (*SQLFetchRow)(struct udm_db_st *db, UDM_SQLRES *R, UDM_PSTR *);
   int    (*SQLStoreResult)(struct udm_db_st *db, UDM_SQLRES *R);
   int    (*SQLFreeResult)(struct udm_db_st *db, UDM_SQLRES *R);
@@ -205,8 +213,14 @@ typedef struct udm_db_st
         UDM_VARLIST       Vars;       /* optional parameters and variables */
 	UDM_SQLDB_HANDLER *sql;
 	void	*specific;
+	void    *ps;                  /* for prepared statements */
 
 	UDM_WORD_CACHE WordCache;
 } UDM_DB;
 
+int UdmSQLPrepareGeneric(UDM_DB *db, const char *query);
+int UdmSQLBindGeneric(UDM_DB *db, int position, void *data, int size, int type);
+int UdmSQLExecGeneric(UDM_DB *db);
+int UdmSQLStmtFreeGeneric(UDM_DB *db);
+
 #endif
