@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,6 +27,28 @@ extern __C_LINK udmhash32_t __UDMCALL UdmHash32(const char * buf, size_t size);
 #define UdmStrHash64(buf)		UdmHash64((buf), strlen(buf))
 #define UdmStrHash32(buf)		UdmHash32((buf), strlen(buf))
 
+typedef struct udm_hash_st
+{
+  void *base;
+  size_t nmemb;
+  size_t size;
+  size_t (*key)(const void *);
+  int (*join)(void *, void *);
+} UDM_HASH;
+
+typedef int (*udm_hash_join)(void*, void*);
+
+void
+UdmHashInit(UDM_HASH *hash, void* base, size_t nmemb, size_t size,
+            size_t(*key)(const void*),
+            int(*join)(void *, void *));
+
+size_t
+UdmHashSize(size_t n);
+
+void
+UdmHashPut(UDM_HASH *hash, void *ptr);
+
 
 
 #endif
