@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -111,8 +111,8 @@ time_t Udm_dp2time_t(const char * time_str);
 extern void UdmTime_t2HttpStr(time_t t, char * time_str);
 /* This one deals with timezone offset */
 extern int UdmInitTZ(void);
-extern unsigned long UdmStartTimer(void);
-
+extern udm_timer_t UdmStartTimer(void);
+extern float UdmStopTimer(udm_timer_t *ticks);
 
 extern char *UdmStrStore (char *dest, const char *src);
 
@@ -159,6 +159,8 @@ extern __C_LINK int __UDMCALL UdmHex2Int(int h);
 extern __C_LINK int __UDMCALL UdmInt2Hex(int i);
 
 
+size_t UdmHexEncode(char *dst, const char *src, size_t len);
+
 #define BASE64_LEN(len) (4 * (((len) + 2) / 3) +2)
 extern __C_LINK size_t __UDMCALL udm_base64_encode (const char *s, char *store, size_t length);
 extern __C_LINK size_t __UDMCALL udm_base64_decode (char * dst, const char * src, size_t len);
@@ -198,6 +200,7 @@ extern void UdmDSTRFree (UDM_DSTR *dstr);
 extern size_t UdmDSTRAppend (UDM_DSTR *dstr, const char *data, size_t size_data);
 extern size_t UdmDSTRAppendHex(UDM_DSTR *dstr, const char *s, size_t slen);
 extern size_t UdmDSTRAppendSTR (UDM_DSTR *dstr, const char *data);
+extern size_t UdmDSTRAppendINT4(UDM_DSTR *s, int i);
 extern int UdmDSTRAppendf (UDM_DSTR *dstr, const char *fmt, ...);
 extern void UdmDSTRReset (UDM_DSTR *dstr);
 extern int UdmDSTRAlloc (UDM_DSTR *dstr, size_t size_data);
@@ -217,6 +220,13 @@ extern void udm_zint4_finalize(UDM_ZINT4_STATE *state);
 extern int udm_dezint4(const char *buf, int4 *array, int buf_len);
 /**************************** end of zint4 stuff *****************************/
 
+#ifdef HAVE_ZLIB
+size_t UdmInflate(char *dst, size_t dstlen, char *src, size_t srclen);
+size_t UdmDeflate(char *dst, size_t dstlen, char *src, size_t srclen);
+#endif
+
+/*****************************************************************************/
+
 extern int udm_l1tolower[256];
 
 #ifdef HAVE_DEBUG
