@@ -1,5 +1,5 @@
 /* $Source: /home/bar/cvsroot/mnogosearch33/php/php_mnogo.c,v $ */
-/* $Id: php_mnogo.c,v 1.26 2007/09/13 08:23:34 bar Exp $ */
+/* $Id: php_mnogo.c,v 1.32 2008/09/03 12:45:11 bar Exp $ */
 
 /*
    +----------------------------------------------------------------------+
@@ -684,7 +684,13 @@ DLEXPORT PHP_FUNCTION(udm_set_agent_param)
         RETURN_FALSE;
       }
       else
+      {
+#if UDM_VERSION_ID >= 30307
+        UdmSynonymListListSortItems(&Agent->Conf->Synonym);
+#else
         UdmSynonymListSort(&(Agent->Conf->Synonyms));
+#endif
+      }
       break;
       
     case UDM_PARAM_QSTRING:
@@ -897,7 +903,7 @@ DLEXPORT PHP_FUNCTION(udm_load_ispell_data)
       if (UdmSpellListListLoad(&Agent->Conf->Spells, err, 128) ||
           UdmAffixListListLoad(&Agent->Conf->Affixes, flags, err, 128))
       {
-        php_error_docref(NULL TSRMLS_CC, E_WARNING,"Error loading ispell data");
+        php_error_docref(NULL TSRMLS_CC, E_WARNING,"Error loading ispell data '%s'", err);
         RETURN_FALSE;
       }
     }
@@ -1248,6 +1254,7 @@ DLEXPORT PHP_FUNCTION(udm_set_agent_param_ex)
           UdmServerInit(&Srv);
           bzero((void*) &Cfg, sizeof(Cfg));
           Cfg.Indexer= Agent;
+          Cfg.flags= UDM_FLAG_LOAD_LANGMAP + UDM_FLAG_SPELL;
           UdmEnvAddLine(&Cfg, var);
           UdmServerFree(&Srv);
           RETURN_TRUE;
@@ -1283,7 +1290,15 @@ DLEXPORT PHP_FUNCTION(udm_set_agent_param_ex)
     if (UdmSynonymListLoad(Agent->Conf,val)) {
       php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s",Agent->Conf->errstr);
       RETURN_FALSE;
-    } else UdmSynonymListSort(&(Agent->Conf->Synonyms));
+    }
+    else
+    {
+#if UDM_VERSION_ID >= 30307
+      UdmSynonymListListSortItems(&Agent->Conf->Synonym);
+#else
+      UdmSynonymListSort(&(Agent->Conf->Synonyms));
+#endif
+    }
   } else if (!strcasecmp(var,"Stopwordfile")) {
     if (UdmStopListLoad(Agent->Conf,val)) {
       php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", Agent->Conf->errstr);
@@ -1590,7 +1605,9 @@ DLEXPORT PHP_FUNCTION(udm_find)
   if ((yyquery) && (strlen(Z_STRVAL_PP(yyquery)))) 
       UdmVarListReplaceStr(&Agent->Conf->Vars, "q",
           Z_STRVAL_PP(yyquery));
-  if ((Res=UdmFind(Agent))) {
+  if ((UDM_OK == UdmEnvPrepare(Agent->Conf)) &&
+      (Res= UdmFind(Agent)))
+  {
       ZEND_REGISTER_RESOURCE(return_value,Res,le_res);
   } else {
       RETURN_FALSE;
