@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -411,6 +411,8 @@ static int add_alias(void *Cfg, size_t ac,char **av)
       Alias.match_type=UDM_MATCH_REGEX;
     else if(!strcasecmp(av[i],"regexp"))
       Alias.match_type=UDM_MATCH_REGEX;
+    else if(!strcasecmp(av[i],"prefix"))
+      Alias.match_type=UDM_MATCH_BEGIN;
     else if(!strcasecmp(av[i],"case"))
       Alias.case_sense= UDM_CASE_INSENSITIVE;
     else if(!strcasecmp(av[i],"nocase"))
@@ -566,7 +568,9 @@ static int add_section_filter(void *Cfg, size_t ac,char **av)
   return UDM_OK;
 }
 
-static int add_type(void *Cfg, size_t ac,char **av)
+
+static int
+add_type_internal(void *Cfg, size_t ac,char **av, UDM_MATCHLIST *Lst)
 {
   UDM_CFG    *C=(UDM_CFG*)Cfg;
   UDM_ENV    *Conf=C->Indexer->Conf;
@@ -602,7 +606,7 @@ static int add_type(void *Cfg, size_t ac,char **av)
     else
     {
       M.pattern=av[i];
-      if(UDM_OK != (rc = UdmMatchListAdd(NULL, &Conf->MimeTypes,&M,
+      if(UDM_OK != (rc = UdmMatchListAdd(NULL, Lst ,&M,
                                          err,sizeof(err), 0)))
       {
         udm_snprintf(Conf->errstr,sizeof(Conf->errstr)-1,"%s",err);
@@ -613,6 +617,25 @@ static int add_type(void *Cfg, size_t ac,char **av)
   return rc;
 }
 
+
+static int
+add_type(void *Cfg, size_t ac,char **av)
+{
+  UDM_CFG    *C=(UDM_CFG*)Cfg;
+  UDM_ENV    *Conf=C->Indexer->Conf;
+  return add_type_internal(Cfg, ac, av, &Conf->MimeTypes);
+}
+
+
+static int
+add_encoding(void *Cfg, size_t ac,char **av)
+{
+  UDM_CFG    *C=(UDM_CFG*)Cfg;
+  UDM_ENV    *Conf=C->Indexer->Conf;
+  return add_type_internal(Cfg, ac, av, &Conf->Encodings);
+}
+
+
 static int add_parser(void *Cfg, size_t ac,char **av)
 {
   UDM_CFG  *C=(UDM_CFG*)Cfg;
@@ -982,6 +1005,30 @@ static int add_user_score(void *Cfg, size_t ac,char **av)
   return UDM_OK;
 }
 
+
+static int
+add_user_site_score(void *Cfg, size_t ac,char **av)
+{
+  UDM_CFG  *C=(UDM_CFG*)Cfg;
+  UDM_ENV  *Conf=C->Indexer->Conf;
+  char name[128];
+  udm_snprintf(name, sizeof(name), "SiteScore.%s", av[1]);
+  UdmVarListReplaceStr(&Conf->Vars, name, av[2]);
+  return UDM_OK;
+}
+
+
+static int add_user_order(void *Cfg, size_t ac,char **av)
+{
+  UDM_CFG *C=(UDM_CFG*) Cfg;
+  UDM_ENV *Conf= C->Indexer->Conf;
+  char name[128];
+  udm_snprintf(name, sizeof(name), "Order.%s", av[1]);
+  UdmVarListReplaceStr(&Conf->Vars, name, av[2]);
+  return UDM_OK;
+}
+
+
 static int flush_srv_table(void *Cfg, size_t ac,char **av)
 {
   UDM_CFG  *C=(UDM_CFG*)Cfg;
@@ -1143,9 +1190,13 @@ static int env_rpl_var(void *Cfg, size_t ac,char **av)
   UDM_ENV  *Conf=C->Indexer->Conf;
   if(!strcasecmp(av[0],"DBAddr"))
   {
-    if(UDM_OK != UdmDBListAdd(&Conf->dbl, av[1] ? av[1] : "", UDM_OPEN_MODE_WRITE))
+    int rc;
+    if(UDM_OK != (rc= UdmDBListAdd(&Conf->dbl, av[1] ? av[1] : "",
+                                   UDM_OPEN_MODE_WRITE)))
     {
-      sprintf(Conf->errstr, "Invalid DBAddr: '%s'",av[1]?av[1]:"");
+      sprintf(Conf->errstr, "%s DBAddr: '%s'",
+              rc == UDM_UNSUPPORTED ? "Unsupported" : "Invalid",
+              av[1]?av[1]:"");
       return UDM_ERROR;
     }
   }
@@ -1162,6 +1213,8 @@ static int env_rpl_var(void *Cfg, size_t ac,char **av)
 #endif
     if (!strcasecmp(av[1], "Freq"))
       seg= 1;
+    if (!strcasecmp(av[1], "CJK"))
+      seg= 1;
     if (!seg)
     {
       sprintf(Conf->errstr, "Unsupported segmenter method: '%s'", av[1]);
@@ -1320,180 +1373,187 @@ typedef struct conf_cmd_st
 
 static UDM_CONFCMD commands[] = 
 {
-  {"Include",              1,1,   do_include},
+  {"Include",              1,1,   do_include},      /* Documented */
 
-  {"ImportEnv",            1,1,   env_rpl_env_var},
-  {"DBAddr",               1,1,   env_rpl_var},
-  {"DefaultContentType",   1,1,   env_rpl_var},
-  {"ResultContentType",    1,1,   env_rpl_var},
+  {"ImportEnv",            1,1,   env_rpl_env_var}, /* Documented */
+  {"DBAddr",               1,1,   env_rpl_var},     /* Documented */
+  {"DefaultContentType",   1,1,   env_rpl_var},     /* Documented */
+  {"ResultContentType",    1,1,   env_rpl_var},     /* Documented */
   {"Listen",               1,1,   env_rpl_var},
-  {"UseRemoteContentType", 1,1,   env_rpl_bool_var},
-  {"UseCRC32URLId",        1,1,   env_rpl_var},
-  {"UseCRC32SiteId",       1,1,   env_rpl_var},
-  {"NewsExtensions",       1,1,   env_rpl_var},
-  {"CrossWords",           1,1,   env_rpl_var},
-  {"SyslogFacility",       1,1,   env_rpl_var},
-  {"AliasProg",            1,1,   env_rpl_var},
-  {"ForceIISCharset1251",  1,1,   env_rpl_var},
-  {"GroupBySite",          1,1,   env_rpl_bool_var},
-  {"Cache",                1,1,   env_rpl_var},
-  {"wf",                   1,1,   env_rpl_var},
-  {"StrictModeThreshold",  1,1,   env_rpl_var},
-  {"PopRankSkipSameSite",  1,1,   env_rpl_var},
-  {"PopRankFeedBack",      1,1,   env_rpl_var},
-  {"PopRankUseTracking",   1,1,   env_rpl_var},
-  {"PopRankUseShowCnt",    1,1,   env_rpl_var},
-  {"PopRankShowCntRatio",  1,1,   env_rpl_var},
-  {"PopRankShowCntWeight", 1,1,   env_rpl_var},
-  {"VarDir",               1,1,   env_rpl_var},
-  {"DocMemCacheSize",      1,1,   env_rpl_var},
-  {"IspellUsePrefixes",    1,1,   env_rpl_var},
-  {"URLSelectCacheSize",   1,1,   env_rpl_var},
-  {"MaxDocSize",           1,1,   env_rpl_var},
-  {"ParserTimeOut",        1,1,   env_rpl_var},
-  {"NumSections",          1,1,   env_rpl_var},
-  {"DateFormat",           1,1,   env_rpl_var},
-  {"GuesserUseMeta",       1,1,   env_rpl_var},
-  {"ResultsLimit",         1,1,   env_rpl_var},
+  {"UseRemoteContentType", 1,1,   env_rpl_bool_var},/* Documented */
+  {"UseCRC32URLId",        1,1,   env_rpl_var},     /* Documented */
+  {"UseCRC32SiteId",       1,1,   env_rpl_var},     
+  {"NewsExtensions",       1,1,   env_rpl_var},     /* Documented */
+  {"CrossWords",           1,1,   env_rpl_bool_var},/* Documented */
+  {"SyslogFacility",       1,1,   env_rpl_var},     /* Documented */
+  {"AliasProg",            1,1,   env_rpl_var},     /* Documented */
+  {"ForceIISCharset1251",  1,1,   env_rpl_var},     /* Documented */
+  {"GroupBySite",          1,1,   env_rpl_bool_var},/* Documented */
+  {"Cache",                1,1,   env_rpl_var},     /* Documented */
+  {"wf",                   1,1,   env_rpl_var},     /* Documented */
+  {"StrictModeThreshold",  1,1,   env_rpl_var},     /* Documented */
+  {"PopRankSkipSameSite",  1,1,   env_rpl_var},     /* Documented */
+  {"PopRankFeedBack",      1,1,   env_rpl_var},     /* Documented */
+  {"PopRankUseTracking",   1,1,   env_rpl_var},     /* Documented */
+  {"PopRankUseShowCnt",    1,1,   env_rpl_var},     /* Documented */
+  {"PopRankShowCntRatio",  1,1,   env_rpl_var},     /* Documented */
+  {"PopRankShowCntWeight", 1,1,   env_rpl_var},     /* Documented */
+  {"VarDir",               1,1,   env_rpl_var},     /* Documented */
+  {"DocMemCacheSize",      1,1,   env_rpl_var},     /* Documented */
+  {"IspellUsePrefixes",    1,1,   env_rpl_var},     /* Documented */
+  {"URLSelectCacheSize",   1,1,   env_rpl_var},     /* Documented */
+  {"URLSelectSkipLock",    1,1,   env_rpl_bool_var},/* Documented */
+  {"MaxDocSize",           1,1,   env_rpl_var},     /* Documented */
+  {"ParserTimeOut",        1,1,   env_rpl_var},     /* Documented */
+  {"NumSections",          1,1,   env_rpl_var},     /* Documented */
+  {"DateFormat",           1,1,   env_rpl_var},     /* Documented */
+  {"GuesserUseMeta",       1,1,   env_rpl_var},     /* Documented */
+  {"ResultsLimit",         1,1,   env_rpl_var},     /* Documented */
   {"Segmenter",            1,1,   env_rpl_var},
-  {"HlBeg",                1,1,   env_rpl_var},
-  {"HlEnd",                1,1,   env_rpl_var},
-  {"Log2stderr",           1,1,   env_rpl_var},
-  {"PagesPerScreen",       1,1,   env_rpl_var},
+  {"HlBeg",                1,1,   env_rpl_var},     /* Documented */
+  {"HlEnd",                1,1,   env_rpl_var},     /* Documented */
+  {"Log2stderr",           1,1,   env_rpl_var},     /* Documented */
+  {"PagesPerScreen",       1,1,   env_rpl_var},     /* Documented */
   {"SQLClearDBHook",       1,1,   env_rpl_var},
+  {"UserCacheQuery",       1,1,   env_rpl_var},     /* Documented */
 #ifdef HAVE_SETVBUF
   {"StdoutBufferSize",     1,1,   env_rpl_var},
 #endif
-  {"AlwaysFoundWord",      1,1,   env_rpl_var},
-  {"CustomLog",            1,1,   env_rpl_var},
+  {"AlwaysFoundWord",      1,1,   env_rpl_var},     /* Documented */
+  {"CustomLog",            1,1,   env_rpl_var},     /* Documented */
   {"CurrentTime",          1,1,   env_rpl_var},
-  {"Locale",               1,1,   env_rpl_var},
-  {"WordDistanceWeight",   1,1,   env_rpl_num_var},
+  {"Locale",               1,1,   env_rpl_var},     /* Documented */
+  {"WordDistanceWeight",   1,1,   env_rpl_num_var}, /* Documented */
   {"MaxCoordFactor",       1,1,   env_rpl_num_var},
-  {"MinCoordFactor",       1,1,   env_rpl_num_var},
-  {"NumWordFactor",        1,1,   env_rpl_var},
-  {"NumDistinctWordFactor",1,1,   env_rpl_num_var},
-  {"UserScoreFactor",      1,1,   env_rpl_num_var},
-  {"WordDensityFactor"    ,1,1,   env_rpl_num_var},
-  {"WordFormFactor"       ,1,1,   env_rpl_num_var},
-  {"URLDataThreshold",     1,1,   env_rpl_num_var},
-  {"DocSizeWeight",        1,1,   env_rpl_num_var},
+  {"MinCoordFactor",       1,1,   env_rpl_num_var}, /* Documented */
+  {"NumWordFactor",        1,1,   env_rpl_var},     /* Documented */
+  {"NumDistinctWordFactor",1,1,   env_rpl_num_var}, /* Documented */
+  {"UserScoreFactor",      1,1,   env_rpl_num_var}, /* Documented */
+  {"WordDensityFactor"    ,1,1,   env_rpl_num_var}, /* Documented */
+  {"WordFormFactor"       ,1,1,   env_rpl_num_var}, /* Documented */
+  {"URLDataThreshold",     1,1,   env_rpl_num_var}, /* Documented */
+  {"DocSizeWeight",        1,1,   env_rpl_num_var}, /* Documented */
   {"RelevancyFactor",      1,1,   env_rpl_num_var},
-  {"DateFactor",           1,1,   env_rpl_num_var},
-  {"MinWordLength",        1,1,   env_rpl_num_var},
-  {"MaxWordLength",        1,1,   env_rpl_num_var},
-  {"SubstringMatchMinWordLength", 1,1, env_rpl_num_var},
-  {"ExcerptSize",          1,1,   env_rpl_num_var},
-  {"ExcerptPadding",       1,1,   env_rpl_num_var},
-  {"LogLevel",             1,1,   env_rpl_num_var},
-  {"WordCacheSize",        1,1,   env_rpl_num_var},
-  {"CVSIgnore",            1,1,   env_rpl_bool_var},
+  {"DateFactor",           1,1,   env_rpl_num_var}, /* Documented */
+  {"MinWordLength",        1,1,   env_rpl_num_var}, /* Documented */
+  {"MaxWordLength",        1,1,   env_rpl_num_var}, /* Documented */
+  {"SubstringMatchMinWordLength", 1,1, env_rpl_num_var}, /* Documented */
+  {"ExcerptSize",          1,1,   env_rpl_num_var}, /* Documented */
+  {"ExcerptPadding",       1,1,   env_rpl_num_var}, /* Documented */
+  {"LogLevel",             1,1,   env_rpl_num_var}, /* Documented */
+  {"CrawlerThreads",       1,1,   env_rpl_num_var}, /* Documented */
+  {"WordCacheSize",        1,1,   env_rpl_num_var}, /* Documented */
+  {"CVSIgnore",            1,1,   env_rpl_bool_var},/* Documented */
   {"UseHTDBURLId",         1,1,   env_rpl_bool_var},
-  {"Suggest",              1,1,   env_rpl_bool_var},
-  {"IndexTime",            1,1,   env_rpl_bool_var},
-  {"ExcerptStopword",      1,1,   env_rpl_bool_var},
-  {"UseCookie",            1,1,   env_rpl_bool_var},
-  {"UseNumericOperators",  1,1,   env_rpl_bool_var},
-  {"SaveSectionSize",      1,1,   env_rpl_bool_var},
-  {"Dehyphenate",          1,1,   env_rpl_bool_var},
-  {"StripAccents",         1,1,   env_rpl_bool_var},
-  {"LoadURLInfo",          1,1,   env_rpl_bool_var},
-  {"LoadTagInfo",          1,1,   env_rpl_bool_var},
+  {"Suggest",              1,1,   env_rpl_bool_var},/* Documented */
+  {"IndexTime",            1,1,   env_rpl_bool_var},/* Documented */
+  {"ExcerptStopword",      1,1,   env_rpl_bool_var},/* Documented */
+  {"UseCookie",            1,1,   env_rpl_bool_var},/* Documented */
+  {"UseNumericOperators",  1,1,   env_rpl_bool_var},/* Documented */
+  {"SaveSectionSize",      1,1,   env_rpl_bool_var},/* Documented */
+  {"Dehyphenate",          1,1,   env_rpl_bool_var},/* Documented */
+  {"StripAccents",         1,1,   env_rpl_bool_var},/* Documented */
+  {"LoadURLInfo",          1,1,   env_rpl_bool_var},/* Documented */
+  {"LoadTagInfo",          1,1,   env_rpl_bool_var},/* Documented */
 
-  {"ReplaceVar",           2,2,   env_rpl_named_var},
+  {"ReplaceVar",           2,2,   env_rpl_named_var},/* Documented */
 
-  {"LocalCharset",         1,1,   env_rpl_charset},
-  {"BrowserCharset",       1,1,   env_rpl_charset},
-  {"CaseFolding",          1,1,   env_rpl_casefolding},
+  {"LocalCharset",         1,1,   env_rpl_charset},  /* Documented */
+  {"BrowserCharset",       1,1,   env_rpl_charset},  /* Documented */
+  {"CaseFolding",          1,1,   env_rpl_casefolding},/* Documented */
 
   {"XMLEnterHook",         2,2,   rpl_xml_hook},
   {"XMLLeaveHook",         2,2,   rpl_xml_hook},
   {"XMLDataHook",          2,2,   rpl_xml_hook},
   
-  {"R0",                   1,1,   env_rpl_rand},
-  {"R1",                   1,1,   env_rpl_rand},
-  {"R2",                   1,1,   env_rpl_rand},
-  {"R3",                   1,1,   env_rpl_rand},
-  {"R4",                   1,1,   env_rpl_rand},
-  {"R5",                   1,1,   env_rpl_rand},
-  {"R6",                   1,1,   env_rpl_rand},
-  {"R7",                   1,1,   env_rpl_rand},
-  {"R8",                   1,1,   env_rpl_rand},
-  {"R9",                   1,1,   env_rpl_rand},
+  {"R0",                   1,1,   env_rpl_rand},     /* Documented */
+  {"R1",                   1,1,   env_rpl_rand},     /* Documented */
+  {"R2",                   1,1,   env_rpl_rand},     /* Documented */
+  {"R3",                   1,1,   env_rpl_rand},     /* Documented */
+  {"R4",                   1,1,   env_rpl_rand},     /* Documented */
+  {"R5",                   1,1,   env_rpl_rand},     /* Documented */
+  {"R6",                   1,1,   env_rpl_rand},     /* Documented */
+  {"R7",                   1,1,   env_rpl_rand},     /* Documented */
+  {"R8",                   1,1,   env_rpl_rand},     /* Documented */
+  {"R9",                   1,1,   env_rpl_rand},     /* Documented */
   
-  {"HTDBAddr",             1,1,   srv_rpl_var},
-  {"HTDBList",             1,1,   srv_rpl_var},
-  {"HTDBDoc",              1,1,   srv_rpl_var},
-  {"HTDBLimit",            1,1,   srv_rpl_var},
+  {"HTDBAddr",             1,1,   srv_rpl_var},      /* Documented */
+  {"HTDBList",             1,1,   srv_rpl_var},      /* Documented */
+  {"HTDBDoc",              1,1,   srv_rpl_var},      /* Documented */
+  {"HTDBLimit",            1,1,   srv_rpl_var},      /* Documented */
   {"SQLImportSection",     1,1,   srv_rpl_var},
   {"SQLExportHref",        1,1,   srv_rpl_var},
-  {"SQLWordForms",         1,1,   env_rpl_var},
-  {"DefaultLang",          1,1,   srv_rpl_var},
-  {"Category",             1,1,   srv_rpl_category},
-  {"Tag",                  1,1,   srv_rpl_var},
-  {"Proxy",                1,1,   srv_rpl_var},
-  {"VaryLang",             1,1,   srv_rpl_var},
+  {"SQLWordForms",         1,1,   env_rpl_var},      /* Documented */
+  {"DefaultLang",          1,1,   srv_rpl_var},      /* Documented */
+  {"Category",             1,1,   srv_rpl_category}, /* Documented */
+  {"Tag",                  1,1,   srv_rpl_var},      /* Documented */
+  {"Proxy",                1,1,   srv_rpl_var},      /* Documented */
+  {"VaryLang",             1,1,   srv_rpl_var},      /* Documented */
   {"UseRobotsTxtURL",      1,1,   srv_rpl_var},
-  {"MaxNetErrors",         1,1,   srv_rpl_num_var},
-  {"MaxHops",              1,1,   srv_rpl_num_var},
-  {"StartHops",            1,1,   srv_rpl_num_var},
-  {"MaxDocPerSite",        1,1,   srv_rpl_num_var},
-  {"ServerWeight",         1,1,   srv_rpl_num_var},
-  {"Robots",               1,1,   srv_rpl_bool_var},
-  {"DetectClones",         1,1,   srv_rpl_bool_var},
-  {"CollectLinks",         1,1,   srv_rpl_bool_var},
-  {"Index",                1,1,   srv_rpl_bool_var},
-  {"FollowSymLinks",       1,1,   srv_rpl_bool_var},
-  {"NetErrorDelayTime",    1,1,   srv_rpl_time_var},
-  {"ReadTimeOut",          1,1,   srv_rpl_time_var},
-  {"DocTimeOut",           1,1,   srv_rpl_time_var},
-  {"Period",               1,1,   srv_rpl_time_var},
-  {"HoldBadHrefs",         1,1,   srv_rpl_time_var},
-  {"HTTPHeader",           1,2,   srv_rpl_hdr},
-  {"ProxyAuthBasic",       1,1,   srv_rpl_auth},
-  {"AuthBasic",            1,1,   srv_rpl_auth},
-  {"MirrorRoot",           1,1,   srv_rpl_mirror},
-  {"MirrorHeadersRoot",    1,1,   srv_rpl_mirror},
-  {"MirrorPeriod",         1,1,   srv_rpl_mirror},
-  {"RemoteCharset",        1,1,   srv_rpl_charset},
-  {"RemoteFileNameCharset",1,1,   srv_rpl_charset},
+  {"MaxNetErrors",         1,1,   srv_rpl_num_var},  /* Documented */
+  {"CrawlDelay",           1,1,   srv_rpl_num_var},  /* Documented */
+  {"MaxHops",              1,1,   srv_rpl_num_var},  /* Documented */
+  {"StartHops",            1,1,   srv_rpl_num_var},  /* Documented */
+  {"MaxDocPerSite",        1,1,   srv_rpl_num_var},  /* Documented */
+  {"ServerWeight",         1,1,   srv_rpl_num_var},  /* Documented */
+  {"Robots",               1,1,   srv_rpl_bool_var}, /* Documented */
+  {"DetectClones",         1,1,   srv_rpl_bool_var}, /* Documented */
+  {"CollectLinks",         1,1,   srv_rpl_bool_var}, /* Documented */
+  {"Index",                1,1,   srv_rpl_bool_var}, /* Documented */
+  {"FollowSymLinks",       1,1,   srv_rpl_bool_var}, /* Documented */
+  {"NetErrorDelayTime",    1,1,   srv_rpl_time_var}, /* Documented */
+  {"ReadTimeOut",          1,1,   srv_rpl_time_var}, /* Documented */
+  {"DocTimeOut",           1,1,   srv_rpl_time_var}, /* Documented */
+  {"Period",               1,1,   srv_rpl_time_var}, /* Documented */
+  {"HoldBadHrefs",         1,1,   srv_rpl_time_var}, /* Documented */
+  {"HTTPHeader",           1,2,   srv_rpl_hdr},      /* Documented */
+  {"ProxyAuthBasic",       1,1,   srv_rpl_auth},     /* Documented */
+  {"AuthBasic",            1,1,   srv_rpl_auth},     /* Documented */
+  {"MirrorRoot",           1,1,   srv_rpl_mirror},   /* Documented */
+  {"MirrorHeadersRoot",    1,1,   srv_rpl_mirror},   /* Documented */
+  {"MirrorPeriod",         1,1,   srv_rpl_mirror},   /* Documented */
+  {"RemoteCharset",        1,1,   srv_rpl_charset},  /* Documented */
+  {"RemoteFileNameCharset",1,1,   srv_rpl_charset},  /* Documented */
   
-  {"Disallow",             1,100, add_filter},
-  {"Allow",                1,100, add_filter},
-  {"CheckMP3Only",         1,100, add_filter},
-  {"CheckMP3",             1,100, add_filter},
-  {"CheckOnly",            1,100, add_filter},
-  {"HrefOnly",             1,100, add_filter},
+  {"Disallow",             1,100, add_filter},       /* Documented */
+  {"Allow",                1,100, add_filter},       /* Documented */
+  {"CheckMP3Only",         1,100, add_filter},       /* Documented */
+  {"CheckMP3",             1,100, add_filter},       /* Documented */
+  {"CheckOnly",            1,100, add_filter},       /* Documented */
+  {"HrefOnly",             1,100, add_filter},       /* Documented */
   {"ImportOnly",           1,100, add_filter},
-  {"Skip",                 1,100, add_filter},
+  {"Skip",                 1,100, add_filter},       /* Documented */
 
-  {"IndexIf",              1,100, add_section_filter},
-  {"NoIndexIf",            1,100, add_section_filter},
-  {"SkipIf",               1,100, add_section_filter},
+  {"IndexIf",              1,100, add_section_filter},/* Documented */
+  {"NoIndexIf",            1,100, add_section_filter},/* Documented */
+  {"SkipIf",               1,100, add_section_filter},/* Documented */
   
-  {"Server",               1,100, add_srv},
-  {"Realm",                1,100, add_srv},
-  {"Subnet",               1,100, add_srv},
-  {"URL",                  1,1,   add_url},
+  {"Server",               1,100, add_srv},          /* Documented */
+  {"Realm",                1,100, add_srv},          /* Documented */
+  {"Subnet",               1,100, add_srv},          /* Documented */
+  {"URL",                  1,1,   add_url},          /* Documented */
   
-  {"Alias",                1,100, add_alias},
-  {"ReverseAlias",         1,100, add_alias},
+  {"Alias",                1,100, add_alias},        /* Documented */
+  {"ReverseAlias",         1,100, add_alias},        /* Documented */
   
-  {"AddType",              1,100, add_type},
-  {"Mime",                 2,4,   add_parser},
-  {"Section",              3,9,   add_section},
-  {"Affix",                3,3,   add_affix},
-  {"Spell",                3,3,   add_spell},
-  {"StopwordFile",         1,1,   add_stoplist},
-  {"LangMapFile",          1,1,   add_langmap},
-  {"LangMapUpdate",        1,1,   env_rpl_var},
-  {"Synonym",              1,1,   add_synonym},
-  {"LoadChineseList",      0,2,   add_chinese},
-  {"LoadThaiList",         0,2,   add_thai},
-  {"Limit",                1,2,   add_limit},
-  {"UserScore",            1,2,   add_user_score},
-  {"ServerTable",          1,1,   add_srv_table},
-  {"FlushServerTable",     0,0,   flush_srv_table},
+  {"AddType",              1,100, add_type},         /* Documented */
+  {"AddEncoding",          1,100, add_encoding},     /* Documented */
+  {"Mime",                 2,4,   add_parser},       /* Documented */
+  {"Section",              3,9,   add_section},      /* Documented */
+  {"Affix",                3,3,   add_affix},        /* Documented */
+  {"Spell",                3,3,   add_spell},        /* Documented */
+  {"StopwordFile",         1,1,   add_stoplist},     /* Documented */
+  {"LangMapFile",          1,1,   add_langmap},      /* Documented */
+  {"LangMapUpdate",        1,1,   env_rpl_var},      /* Documented */
+  {"Synonym",              1,1,   add_synonym},      /* Documented */
+  {"LoadChineseList",      0,2,   add_chinese},      /* Documented */
+  {"LoadThaiList",         0,2,   add_thai},         /* Documented */
+  {"Limit",                1,2,   add_limit},        /* Documented */
+  {"UserScore",            1,2,   add_user_score},   /* Documented */
+  {"UserSiteScore",        1,2,   add_user_site_score},/* Documented */
+  {"UserOrder",            2,2,   add_user_order},   /* Documented */
+  {"ServerTable",          1,1,   add_srv_table},    /* Documented */
+  {"FlushServerTable",     0,0,   flush_srv_table},  /* Documented */
   {"DBListFree",           0,0,   dblist_free},
   
   {NULL,0,0,0}
@@ -1941,7 +2001,7 @@ UdmAffixListListPrint(FILE *f, UDM_AFFIXLISTLIST *L)
   {
     UDM_AFFIXLIST *A= &L->Item[i];
     udm_snprintf(str, sizeof(str),
-                 "Affix %s %s %s", A->lang, A->cset, A->fname);
+                 "Affix %s %s '%s'", A->lang, A->cset, A->fname);
     fprintf(f, "%s\n", str);
   }
   return UDM_OK;
@@ -1957,7 +2017,7 @@ UdmSpellListListPrint(FILE *f, UDM_SPELLLISTLIST *L)
   {
     UDM_SPELLLIST *S= &L->Item[i];
     udm_snprintf(str, sizeof(str),
-                 "Spell %s %s %s", S->lang, S->cset, S->fname);
+                 "Spell %s %s '%s'", S->lang, S->cset, S->fname);
     fprintf(f, "%s\n", str);
   }
   return UDM_OK;
@@ -1973,7 +2033,7 @@ UdmLangmapListPrint(FILE *f, UDM_LANGMAPLIST *L)
   {
     UDM_LANGMAP *M= &L->Map[i];
     udm_snprintf(str, sizeof(str),
-                 "Langmap %s", M->filename);
+                 "LangmapFile '%s'", M->filename);
     fprintf(f, "%s\n", str);
   }
   return UDM_OK;
@@ -1989,7 +2049,7 @@ UdmStopListListPrint(FILE *f, UDM_STOPLISTLIST *L)
   {
     UDM_STOPLIST *S= &L->Item[i];
     udm_snprintf(str, sizeof(str),
-                 "StopwordFile %s", S->fname);
+                 "StopwordFile '%s'", S->fname);
     fprintf(f, "%s\n", str);
   }
   return UDM_OK;
@@ -2005,7 +2065,7 @@ UdmSynonymListListPrint(FILE *f, UDM_SYNONYMLISTLIST *L)
   {
     UDM_SYNONYMLIST *S= &L->Item[i];
     udm_snprintf(str, sizeof(str),
-                 "Synonym %s", S->fname);
+                 "Synonym '%s'", S->fname);
     fprintf(f, "%s\n", str);
   }
   return UDM_OK;
