@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -54,10 +54,10 @@
 
 
 #include "udm_common.h"
+#include "udm_utils.h"
 #include "udm_db.h"
 #include "udm_db_int.h"
 #include "udm_sqldbms.h"
-#include "udm_utils.h"
 #include "udm_url.h"
 #include "udm_sdp.h"
 #include "udm_vars.h"
@@ -137,6 +137,15 @@ int UdmURLData(UDM_ENV *Conf, UDM_URLDATALIST *L, UDM_DB *db)
 }
 */
 
+
+int
+UdmDBIsActive(UDM_AGENT *A, size_t num_arg)
+{
+  size_t num= UdmVarListFindInt(&A->Conf->Vars, "DBLimit", 0);
+  return num == 0 || num == num_arg + 1;
+}
+
+
 __C_LINK int __UDMCALL UdmClearDatabase(UDM_AGENT *A)
 {
   int  res=UDM_ERROR;
@@ -145,6 +154,8 @@ __C_LINK int __UDMCALL UdmClearDatabase(UDM_AGENT *A)
 
   for (i = 0; i < dbto; i++)
   {
+    if (!UdmDBIsActive(A, i))
+      continue;
     db = &A->Conf->dbl.db[i];
 #ifdef HAVE_SQL
     res = UdmClearDBSQL(A, db);
@@ -513,19 +524,21 @@ __C_LINK int __UDMCALL UdmURLAction(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd)
 __C_LINK int __UDMCALL UdmTargets(UDM_AGENT *A)
 {
   int  res=UDM_ERROR;
-  UDM_DB  *db;
   size_t i, dbfrom = 0, dbto;
 
   UDM_LOCK_CHECK_OWNER(A, UDM_LOCK_CONF);
   dbto =  A->Conf->dbl.nitems;
   UdmResultFree(&A->Conf->Targets);
 
-  for (i = dbfrom; i < dbto; i++)
+  for (i= dbfrom; i < dbto; i++)
   {
-    db = &A->Conf->dbl.db[i];
+    UDM_DB *db= &A->Conf->dbl.db[i];
+    
+    if (!UdmDBIsActive(A, i))
+      continue;
     UDM_GETLOCK(A, UDM_LOCK_DB);
 #ifdef HAVE_SQL
-    res = UdmTargetsSQL(A, db);
+    res= UdmTargetsSQL(A, db);
 #endif
     if(res != UDM_OK)
     {
@@ -656,9 +669,10 @@ UdmFindWordsDB(UDM_AGENT *A, UDM_RESULT *CurRes, UDM_DB *db,
 {
   int rc= UDM_OK;
   const char *dbaddr= UdmVarListFindStr(&db->Vars,"DBAddr","<noaddr>");
-      
-  UdmLog(A, UDM_LOG_DEBUG, "UdmFind for %s", dbaddr);
-      
+  udm_timer_t ticks= UdmStartTimer();
+  
+  UdmLog(A, UDM_LOG_DEBUG, "UdmFindWordsDB for %s", dbaddr);
+  
   switch(db->DBDriver)
     {
     case UDM_DB_SEARCHD:
@@ -677,6 +691,8 @@ UdmFindWordsDB(UDM_AGENT *A, UDM_RESULT *CurRes, UDM_DB *db,
       break;
 #endif
   }
+  UdmLog(A, UDM_LOG_DEBUG, "UdmFindWordsDB done: %.2f",
+         UdmStopTimer(&ticks));
   return rc;
 }
 
@@ -712,7 +728,7 @@ UdmResultSetMachineNumber(UDM_URLDATALIST *List, size_t num)
       We use (256-i) to sort a document from the first database
       before the same document from the second database.
     */
-    Item[i].coord = (Item[i].coord << 8) + (255 - (num & 255));
+    Item[i].score = (Item[i].score << 8) + (255 - (num & 255));
    }
   return UDM_OK;
 }
@@ -1033,6 +1049,7 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   size_t  offs=        (size_t) UdmVarListFindInt(&A->Conf->Vars, "offs", 0);
   size_t  ExcerptSize= (size_t)UdmVarListFindInt(&A->Conf->Vars, "ExcerptSize", 256);
   size_t  ExcerptPadding = (size_t)UdmVarListFindInt(&A->Conf->Vars, "ExcerptPadding", 40);
+  int     UseLocalCachedCopy= UdmVarListFindBool(&A->Conf->Vars, "UseLocalCachedCopy", 0);
   size_t  max_offs= 1000; /* Can be a search.htm command, make sure it is not overwritten by query string */
   char    str[128];
 
@@ -1059,12 +1076,12 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   }
 
   ticks_= UdmStartTimer();
-  UdmLog(A, UDM_LOG_DEBUG, "Start FindWords");  
+  UdmLog(A, UDM_LOG_DEBUG, "Start UdmFindWords");  
   if (!Res->WWList.nwords ||
       UDM_OK != (res= UdmFindWords(A, TmpRes, Res, Res->first + page_size)))
     goto conv;
   ticks_= UdmStartTimer() - ticks_;
-  UdmLog(A, UDM_LOG_DEBUG, "Stop  FindWords:\t\t%.2f", (float)ticks_/1000);
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  UdmFindWords:\t\t%.2f", (float)ticks_/1000);
   
   if (!Res->total_found)
   {
@@ -1121,12 +1138,16 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   for(i=0;i<Res->num_rows;i++)
   {
     UDM_URLDATA *Data= &Res->URLData.Item[i + Res->first];
-    uint4 score= Data->coord;
-    UdmVarListReplaceUnsigned(&Res->Doc[i].Sections, "PerSite", Data->per_site);
+    uint4 score= Data->score;
+    UDM_VARLIST *Sections= &Res->Doc[i].Sections;
+    UdmVarListReplaceUnsigned(Sections, "PerSite", Data->per_site);
     udm_snprintf(str, 128, "%.3f%%", ((double)(score >> 8)) / 1000);
-    UdmVarListReplaceStr(&Res->Doc[i].Sections, "Score", str);
-    UdmVarListReplaceInt(&Res->Doc[i].Sections,"Order",(int)(i + Res->first + 1));
-    UdmVarListReplaceInt(&Res->Doc[i].Sections, "dbnum", UDM_COORD2DBNUM(score));
+    UdmVarListReplaceStr(Sections, "Score", str);
+    UdmVarListReplaceInt(Sections,"Order",(int)(i + Res->first + 1));
+    UdmVarListReplaceInt(Sections, "dbnum", UDM_COORD2DBNUM(score));
+    /* Add a fake cached copy to have search.cgi display stored_href */
+    if (UseLocalCachedCopy && !UdmVarListFind(Sections, "CachedCopy"))
+      UdmVarListAddStr(Sections, "CachedCopy", "Fake-CachedCopy");
   }
   
   
@@ -1296,14 +1317,15 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "oracle8", UDM_DB_ORACLE8, UDM_DB_ORACLE8, 1, 
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
-    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_ROWNUM,
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND |
+    UDM_SQL_HAVE_ROWNUM    | UDM_SQL_HAVE_GOOD_COMMIT | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_oracle_handler
   },
   {
     "oracle", UDM_DB_ORACLE8, UDM_DB_ORACLE8, 1, 
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
     UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND |
-    UDM_SQL_HAVE_ROWNUM    | UDM_SQL_HAVE_GOOD_COMMIT,
+    UDM_SQL_HAVE_ROWNUM    | UDM_SQL_HAVE_GOOD_COMMIT | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_oracle_handler
   },
 #endif
@@ -1311,20 +1333,21 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "mssql", UDM_DB_MSSQL, UDM_DB_MSSQL, 1,
     UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_TRUNCATE |
-    UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX,
+    UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_ctlib_handler
   },
   {
     "sybase", UDM_DB_SYBASE, UDM_DB_MSSQL, 1,
     UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_TRUNCATE |
     UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX    |
-    UDM_SQL_HAVE_GOOD_COMMIT,
+    UDM_SQL_HAVE_GOOD_COMMIT | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_ctlib_handler
   },
 #endif
 #if (HAVE_MYSQL)
   { 
     "mysql", UDM_DB_MYSQL, UDM_DB_MYSQL, 1,
+    UDM_SQL_HAVE_BIND  |
     UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_0xHEX,
     &udm_sqldb_mysql_handler
   },
@@ -1332,7 +1355,9 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
 #if (HAVE_PGSQL)
   {
     "pgsql", UDM_DB_PGSQL, UDM_DB_PGSQL, 1,
-    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT,
+    UDM_SQL_HAVE_BIND  |
+    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY |
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_pgsql_handler,
   },
 #endif
@@ -1345,7 +1370,8 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
     are more then 1500. We'd better have to fix code to avoid 
     big INs instead of hidding DBSQL_IN.
     */
-    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND |
+    UDM_SQL_HAVE_FIRST_SKIP | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_ibase_handler,
   },
 #endif
@@ -1353,7 +1379,8 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "sqlite",
     UDM_DB_SQLITE, UDM_DB_SQLITE, 1,
-    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY,
+    UDM_SQL_HAVE_BIND  |
+    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_sqlite_handler,
   },
 #endif
@@ -1361,75 +1388,85 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "sqlite3",
     UDM_DB_SQLITE3, UDM_DB_SQLITE3, 1,
+    UDM_SQL_HAVE_BIND   |
     UDM_SQL_HAVE_LIMIT  | UDM_SQL_HAVE_GROUPBY |
-    UDM_SQL_HAVE_STDHEX | UDM_SQL_HAVE_GOOD_COMMIT, 
+    UDM_SQL_HAVE_STDHEX | UDM_SQL_HAVE_GOOD_COMMIT | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_sqlite3_handler,
   },
 #endif
 #if (HAVE_ODBC)
   {
     "odbc-solid", UDM_DB_SOLID, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-sapdb", UDM_DB_SAPDB, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-db2", UDM_DB_DB2, UDM_DB_ODBC, 1,
     UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND |
-    UDM_SQL_HAVE_BIND    | UDM_SQL_HAVE_STDHEX,
+    UDM_SQL_HAVE_BIND    | UDM_SQL_HAVE_STDHEX | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-access", UDM_DB_ACCESS, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_0xHEX,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT |
+    UDM_SQL_HAVE_0xHEX   | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-mimer", UDM_DB_MIMER, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT |
+    UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_STDHEX | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-cache", UDM_DB_CACHE, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT |
+    UDM_SQL_HAVE_BIND    | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
   { 
     "odbc-virtuoso", UDM_DB_VIRT, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT |
+    UDM_SQL_HAVE_BIND    | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-oracle", UDM_DB_ORACLE8, UDM_DB_ODBC, 1,
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
     UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND |
-    UDM_SQL_HAVE_ROWNUM    | UDM_SQL_HAVE_GOOD_COMMIT,
+    UDM_SQL_HAVE_ROWNUM    | UDM_SQL_HAVE_GOOD_COMMIT |
+    UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-oracle8", UDM_DB_ORACLE8, UDM_DB_ODBC, 1,
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
-    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_ROWNUM,
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_ROWNUM |
+    UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-mssql", UDM_DB_MSSQL, UDM_DB_ODBC, 1,
     UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_TRUNCATE |
-    UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX,
+    UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX |
+    UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-sybase", UDM_DB_SYBASE, UDM_DB_ODBC, 1,
     UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_TRUNCATE |
-    UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX,
+    UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX |
+    UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-mysql", UDM_DB_MYSQL, UDM_DB_ODBC, 1,
+    UDM_SQL_HAVE_BIND  |
     UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_0xHEX,
     &udm_sqldb_odbc_handler,
   },
@@ -1437,7 +1474,8 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
     /* Bind does not seem to work with BYTEA in Windows */
     "odbc-pgsql", UDM_DB_PGSQL, UDM_DB_ODBC, 1,
     UDM_SQL_HAVE_LIMIT      | UDM_SQL_HAVE_GROUPBY |
-    UDM_SQL_HAVE_SUBSELECT  /*| UDM_SQL_HAVE_BIND*/,
+    UDM_SQL_HAVE_SUBSELECT  /*| UDM_SQL_HAVE_BIND*/|
+    UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
   {
@@ -1448,7 +1486,8 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
     are more then 1500. We'd better have to fix code to avoid 
     big INs instead of hidding DBSQL_IN.
     */
-    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND |
+    UDM_SQL_HAVE_FIRST_SKIP | UDM_SQL_HAVE_TRANSACT,
     &udm_sqldb_odbc_handler,
   },
 #endif
@@ -1492,8 +1531,7 @@ int UdmDBSetAddr(UDM_DB *db, const char *dbaddr, int mode)
   
   if((!dbaddr) || UdmURLParse(&addr, dbaddr) || (!addr.schema))
   {
-    /* TODO: add better error message here */
-    rc= UDM_ERROR;
+    rc= UDM_ERROR; /* Invalid DBAddr */
     goto ret;
   }
   
@@ -1548,8 +1586,7 @@ int UdmDBSetAddr(UDM_DB *db, const char *dbaddr, int mode)
     UDM_SQLDB_DRIVER *drv= UdmSQLDriverByName(addr.schema);
     if (!drv)
     {
-    
-      rc= UDM_ERROR;
+      rc= UDM_UNSUPPORTED; /* Unsupported DBAddr */
       goto ret;
     }
     
@@ -1622,6 +1659,35 @@ int UdmDBSetAddr(UDM_DB *db, const char *dbaddr, int mode)
     UdmUnescapeCGIQuery(db->DBName, src);
     UdmFree(src);
   }
+  
+  if (UdmVarListFindInt(&db->Vars, "ps", 0) == 123)
+  {
+    db->sql->SQLPrepare= UdmSQLPrepareGeneric;
+    db->sql->SQLBind= UdmSQLBindGeneric; 
+    db->sql->SQLExec= UdmSQLExecGeneric;
+    db->sql->SQLStmtFree= UdmSQLStmtFreeGeneric;
+    db->flags|= UDM_SQL_HAVE_BIND;
+  }
+  else if ((db->DBType == UDM_DB_MSSQL   ||
+            db->DBType == UDM_DB_SYBASE  ||
+            db->DBType == UDM_DB_MYSQL   ||
+            db->DBType == UDM_DB_PGSQL   ||
+            db->DBType == UDM_DB_SQLITE  ||
+            db->DBType == UDM_DB_SQLITE3)&&
+            !strcasecmp(UdmVarListFindStr(&db->Vars, "ps", ""), "none"))
+  {
+    db->flags&= (0x7FFFFFFF ^ UDM_SQL_HAVE_BIND);
+  }
+  else if ((db->DBDriver == UDM_DB_MYSQL ||
+            db->DBDriver == UDM_DB_PGSQL ||
+            db->DBDriver == UDM_DB_ODBC) &&
+            db->sql->SQLExec &&
+            UdmVarListFindBool(&db->Vars, "ps", 0))
+  {
+    db->flags|= UDM_SQL_HAVE_BIND;
+  }
+
+
 ret:
   UdmURLFree(&addr);
   return rc;
@@ -1641,6 +1707,8 @@ __C_LINK int __UDMCALL UdmStatAction(UDM_AGENT *A, UDM_STATLIST *S)
 
   for (i = dbfrom; i < dbto; i++)
   {
+    if (!UdmDBIsActive(A, i))
+      continue;
     db = &A->Conf->dbl.db[i];
 #ifdef HAVE_SQL
     UDM_GETLOCK(A, UDM_LOCK_DB);
@@ -1790,97 +1858,9 @@ void UdmDBListFree(UDM_DBLIST *List)
   UdmDBListInit(List);
 }
 
-/********************* MultiCache stuff *************************/
-UDM_MULTI_CACHE *UdmMultiCacheInit (UDM_MULTI_CACHE *cache)
-{
-  size_t i;
 
-  if (! cache)
-  {
-    cache = UdmMalloc(sizeof(UDM_MULTI_CACHE));
-    if (! cache) return(NULL);
-    cache->free = 1;
-  }
-  else
-  {
-    cache->free = 0;
-  }
-  cache->nrecs = 0;
+/******************************************/
 
-  for (i = 0; i <= MULTI_DICTS; i++)
-  {
-    cache->tables[i].nurls = 0;
-    cache->tables[i].urls = NULL;
-  }
-
-  cache->nurls = 0;
-  cache->urls = NULL;
-
-  return(cache);
-}
-
-void UdmMultiCacheFree (UDM_MULTI_CACHE *cache)
-{
-  size_t w, s, u, t;
-
-  if (! cache) return;
-  for (t = 0; t <= MULTI_DICTS; t++)
-  {
-    UDM_MULTI_CACHE_TABLE *table = &cache->tables[t];
-    for (u = 0; u < table->nurls; u++)
-    {
-      UDM_MULTI_CACHE_URL *url = &table->urls[u];
-      for (s = 0; s < url->nsections; s++)
-      {
-        UDM_MULTI_CACHE_SECTION *section = &url->sections[s];
-        for (w = 0; w < section->nwords; w++) {
-          UDM_MULTI_CACHE_WORD *word = &section->words[w];
-          UdmFree(word->word);
-          UdmFree(word->intags);
-        }
-        UdmFree(section->words);
-      }
-      UdmFree(url->sections);
-    }
-    UdmFree(table->urls);
-    cache->tables[t].nurls = 0;
-    cache->tables[t].urls = NULL;
-  }
-
-  UdmFree(cache->urls);
-  cache->nurls = 0;
-  cache-> urls = NULL;
-  cache->nrecs = 0;
-
-  if (cache->free) UdmFree(cache);
-}
-
-int UdmWordCacheFlush (UDM_AGENT *Indexer)
-{
-#ifdef HAVE_SQL
-  size_t i;
-  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
-  for (i= 0; i < Indexer->Conf->dbl.nitems; i++)
-  {
-    int rc= UDM_OK;
-    UDM_DB *db= &Indexer->Conf->dbl.db[i];
-    UDM_GETLOCK(Indexer, UDM_LOCK_DB);
-    switch (db->DBMode) {
-      case UDM_DBMODE_MULTI:
-        rc= UdmWordCacheWrite(Indexer, db, 0);
-        break;
-    }
-    if (rc != UDM_OK)
-    {
-      UdmLog(Indexer, UDM_LOG_ERROR, "%s", db->errstr);
-      return rc;
-    }
-    UDM_RELEASELOCK(Indexer, UDM_LOCK_DB);
-  }
-  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
-#endif
-  return UDM_OK;
-}
 
 int UdmMulti2Blob (UDM_AGENT *Indexer)
 {
@@ -1895,6 +1875,8 @@ int UdmMulti2Blob (UDM_AGENT *Indexer)
   {
     int rc;
     UDM_DB *db = &Indexer->Conf->dbl.db[i];
+    if (!UdmDBIsActive(Indexer, i))
+      continue;
     UDM_GETLOCK(Indexer, UDM_LOCK_DB);
     rc= UdmConvert2BlobSQL(Indexer, db);
     UDM_RELEASELOCK(Indexer, UDM_LOCK_DB);
@@ -1912,68 +1894,6 @@ int UdmMulti2Blob (UDM_AGENT *Indexer)
 }
 
 
-int UdmRewriteURL(UDM_AGENT *Indexer)
-{
-#ifdef HAVE_SQL
-  size_t i;
-  unsigned long ticks;
-
-  UdmLog(Indexer,UDM_LOG_ERROR,"Converting to blob");
-  ticks=UdmStartTimer();
-
-  for (i = 0; i < Indexer->Conf->dbl.nitems; i++)
-  {
-    int rc;
-    UDM_DB *db = &Indexer->Conf->dbl.db[i];
-    UDM_GETLOCK(Indexer, UDM_LOCK_DB);
-    rc= UdmBlobWriteURL(Indexer, db, "bdict",
-          UdmVarListFindBool(&db->Vars, "deflate", 0));
-    UDM_RELEASELOCK(Indexer, UDM_LOCK_DB);
-    if (rc != UDM_OK)
-    {
-      UdmLog(Indexer,UDM_LOG_ERROR,"%s",db->errstr); 
-      return rc;
-    }
-  }
-
-  ticks=UdmStartTimer()-ticks;
-  UdmLog(Indexer,UDM_LOG_ERROR,"Converting to blob finished\t%.2f",(float)ticks/1000);
-#endif
-  return UDM_OK;
-}
-
-
-int UdmRewriteLimits(UDM_AGENT *Indexer)
-{
-#ifdef HAVE_SQL
-  size_t i;
-  unsigned long ticks;
-
-  UdmLog(Indexer,UDM_LOG_ERROR,"Rewritting limits");
-  ticks=UdmStartTimer();
-
-  for (i = 0; i < Indexer->Conf->dbl.nitems; i++)
-  {
-    int rc;
-    UDM_DB *db = &Indexer->Conf->dbl.db[i];
-    UDM_GETLOCK(Indexer, UDM_LOCK_DB);
-    rc= UdmBlobWriteLimits(Indexer, db, "bdict",
-          UdmVarListFindBool(&db->Vars, "deflate", 0));
-    UDM_RELEASELOCK(Indexer, UDM_LOCK_DB);
-    if (rc != UDM_OK)
-    {
-      UdmLog(Indexer,UDM_LOG_ERROR,"%s",db->errstr); 
-      return rc;
-    }
-  }
-
-  ticks=UdmStartTimer()-ticks;
-  UdmLog(Indexer,UDM_LOG_ERROR,"Rewritting limits\t%.2f",(float)ticks/1000);
-#endif
-  return UDM_OK;
-}
-
-
 int UdmExport (UDM_AGENT *Indexer)
 {
   int rc= UDM_OK;
@@ -1987,6 +1907,8 @@ int UdmExport (UDM_AGENT *Indexer)
   for (i = 0; i < Indexer->Conf->dbl.nitems; i++)
   {
     UDM_DB *db = &Indexer->Conf->dbl.db[i];
+    if (!UdmDBIsActive(Indexer, i))
+      continue;
     UDM_GETLOCK(Indexer, UDM_LOCK_DB);
     rc= UdmExportSQL(Indexer, db);
     UDM_RELEASELOCK(Indexer, UDM_LOCK_DB);
@@ -2002,413 +1924,3 @@ int UdmExport (UDM_AGENT *Indexer)
 #endif
   return rc;
 }
-
-
-static int mccmpword(uint4 *a, uint4 *b)
-{
-  if (*a > *b) return 1;
-  if (*a < *b) return -1;
-  return 0;
-}
-
-
-static size_t UdmMultiCacheAddWord (UDM_MULTI_CACHE_WORD *cache, uint4 coord)
-{
-  uint4 *tmp;
-
-  if (! cache) return(0);
-  tmp = UdmRealloc(cache->intags, (cache->nintags + 1) * sizeof(uint4));
-  if (! tmp) return(0);
-  cache->intags = tmp;
-  cache->intags[cache->nintags] = coord;
-  cache->nintags++;
-  UdmSort(cache->intags, cache->nintags, sizeof(uint4), (udm_qsort_cmp)mccmpword);
-  return(1);
-}
-
-static size_t UdmMultiCacheAddSection (UDM_MULTI_CACHE_SECTION *cache, UDM_WORD *word)
-{
-  size_t i;
-  uint4 coord = word->pos & 0x1FFFFF;
-
-  if (! cache) return(0);
-
-  for (i = 0; i < cache->nwords; i++)
-  {
-    if (! strcmp(cache->words[i].word, word->word))  break;
-  }
-
-  if (i == cache->nwords)
-  {
-    UDM_MULTI_CACHE_WORD *tmp;
-    tmp = UdmRealloc(cache->words, (cache->nwords + 1) * sizeof(UDM_MULTI_CACHE_WORD));
-    if (! tmp) return(0);
-    cache->words = tmp;
-    cache->words[cache->nwords].word = (char *)UdmStrdup(word->word);
-    cache->words[cache->nwords].nintags = 0;
-    cache->words[cache->nwords].intags = NULL;
-    cache->nwords++;
-  }
-
-  return(UdmMultiCacheAddWord(&cache->words[i], coord));
-}
-
-static size_t UdmMultiCacheAddURL (UDM_MULTI_CACHE_URL *cache, UDM_WORD *word)
-{
-  size_t i;
-  unsigned char secno= word->secno;
-
-  if (! cache) return(0);
-
-  for (i = 0; i < cache->nsections; i++)
-    if (cache->sections[i].secno == secno) break;
-
-  if (i == cache->nsections)
-  {
-    UDM_MULTI_CACHE_SECTION *tmp;
-    tmp = UdmRealloc(cache->sections, (cache->nsections + 1) * sizeof(UDM_MULTI_CACHE_SECTION));
-    if (! tmp) return(0);
-    cache->sections = tmp;
-    cache->sections[cache->nsections].secno = secno;
-    cache->sections[cache->nsections].nwords = 0;
-    cache->sections[cache->nsections].words = NULL;
-    cache->nsections++;
-  }
-
-  return(UdmMultiCacheAddSection(&cache->sections[i], word));
-}
-
-static size_t UdmMultiCacheAddTable (UDM_MULTI_CACHE_TABLE *cache, urlid_t url_id, unsigned char reindex, UDM_WORD *word)
-{
-  size_t i;
-
-  if (! cache) return(0);
-  for (i = 0; i < cache->nurls; i++)
-  {
-    if (cache->urls[i].url_id == url_id) break;
-  }
-
-  if (i == cache->nurls)
-  {
-    UDM_MULTI_CACHE_URL *tmp;
-    tmp = UdmRealloc(cache->urls, (cache->nurls + 1) * sizeof(UDM_MULTI_CACHE_URL));
-    if (! tmp) return(0);
-    cache->urls = tmp;
-    cache->urls[cache->nurls].url_id = url_id;
-    cache->urls[cache->nurls].reindex = reindex;
-    cache->urls[cache->nurls].nsections = 0;
-    cache->urls[cache->nurls].sections = NULL;
-    cache->nurls++;
-  }
-
-  return(UdmMultiCacheAddURL(&cache->urls[i], word));
-}
-
-size_t UdmMultiCacheAdd (UDM_MULTI_CACHE *cache, urlid_t url_id, unsigned char reindex, UDM_WORD *word)
-{
-  udmhash32_t table = UdmStrHash32(word->word) & MULTI_DICTS;
-  size_t i;
-  
-  if (! cache) return(0);
-
-  cache->nrecs++;
-
-  if (reindex)
-  {
-    for (i = 0; i < cache->nurls; i++)
-      if (cache->urls[i] == url_id) break;
-
-    if (i == cache->nurls)
-    {
-      urlid_t *tmp;
-      tmp = UdmRealloc(cache->urls, (cache->nurls + 1) * sizeof(urlid_t));
-      if (! tmp) return(0);
-      cache->urls = tmp;
-      cache->urls[cache->nurls] = url_id;
-      cache->nurls++;
-    }
-  }
-
-  return(UdmMultiCacheAddTable(&cache->tables[table], url_id, reindex, word));
-}
-
-/******************** BlobCache stuff *******************/
-
-UDM_BLOB_CACHE *UdmBlobCacheInit (UDM_BLOB_CACHE *cache)
-{
-  if (! cache)
-  {
-    cache = UdmMalloc(sizeof(UDM_BLOB_CACHE));
-    if (! cache) return(NULL);
-    cache->free= 1;
-  }
-  else
-  {
-    cache->free= 0;
-  }
-  cache->errors= 0;
-  cache->nwords= 0;
-  cache->awords= 0;
-  cache->words= NULL;
-
-  return(cache);
-}
-
-void UdmBlobCacheFree (UDM_BLOB_CACHE *cache)
-{
-  size_t i;
-  for (i= 0; i < cache->nwords; i++)
-  {
-    if (cache->words[i].freeme)
-      UDM_FREE(cache->words[i].word);
-  }
-
-  UdmFree(cache->words);
-  cache->errors= 0;
-  cache->nwords= 0;
-  cache->awords= 0;
-  cache->words= NULL;
-
-  if (cache->free) UdmFree(cache);
-}
-
-
-static inline int
-UdmBlobCacheCheckValue(urlid_t url_id, unsigned char secno,
-                       const char *word, size_t nintags,
-                       const char *intag, size_t intaglen)
-{
-  if (! url_id)
-  {
-    fprintf(stderr, "url_id variable empty\n");
-    return 1;
-  }
-  if (! secno)
-  {
-    fprintf(stderr, "secno variable empty\n");
-    return 1;
-  }
-  if (! word)
-  {
-    fprintf(stderr, "word variable empty\n");
-    return 1;
-  }
-  if (! nintags)
-  {
-    fprintf(stderr, "nintags variable empty\n");
-    return 1;
-  }
-  if (! intag)
-  {
-    fprintf(stderr, "intag variable empty\n");
-    return 1;
-  }
-  return 0;
-}
-
-
-static inline int
-UdmBlobCacheRealloc(UDM_BLOB_CACHE *cache)
-{
-  if (cache->nwords == cache->awords)
-  {
-    UDM_BLOB_CACHE_WORD *tmp;
-    size_t nbytes= (cache->awords + 256) * sizeof(UDM_BLOB_CACHE_WORD);
-    tmp= UdmRealloc(cache->words, nbytes);
-    if (!tmp)
-    {
-      cache->errors++;
-      if (cache->errors < 10 || (cache->errors % 2048) == 0)
-      fprintf(stderr, "BlobCacheRealloc: failed %d times: %d bytes, %d records\n",
-              cache->errors, nbytes, (cache->awords + 256));
-      return 1;
-    }
-    cache->words = tmp;
-    cache->awords += 256;
-  }
-  return 0;
-}
-
-
-size_t UdmBlobCacheAdd (UDM_BLOB_CACHE *cache, urlid_t url_id,
-                        unsigned char secno, const char *word,
-                        size_t nintags, const char *intag, size_t intaglen)
-{
-  size_t word_len;
-  UDM_BLOB_CACHE_WORD *W;
-  
-  if (UdmBlobCacheCheckValue(url_id, secno, word, nintags, intag, intaglen) ||
-      UdmBlobCacheRealloc(cache))
-    return 0;
-  
-  word_len= strlen(word);
-  W= &cache->words[cache->nwords];
-  W->secno= secno;
-  W->url_id= url_id;
-  W->nintags= nintags;
-  W->ntaglen= intaglen;
-  W->word= UdmMalloc(word_len + intaglen + 2);
-  W->intags= W->word + word_len + 1;
-  memcpy(W->word, word, word_len + 1);
-  memcpy(W->intags, intag, intaglen);
-  W->intags[intaglen]= '\0';
-  W->freeme= 1;
-
-  cache->nwords++;
-
-  return(1);
-}
-
-
-size_t UdmBlobCacheAdd2(UDM_BLOB_CACHE *cache, urlid_t url_id,
-                        unsigned char secno, char *word,
-                        size_t nintags, char *intag, size_t intaglen)
-{
-  UDM_BLOB_CACHE_WORD *W;
-  
-  if (UdmBlobCacheCheckValue(url_id, secno, word, nintags, intag, intaglen) ||
-      UdmBlobCacheRealloc(cache))
-    return 0;
-  
-  W= &cache->words[cache->nwords];
-  W->secno= secno;
-  W->url_id= url_id;
-  W->nintags= nintags;
-  W->ntaglen= intaglen;
-  W->word= word;
-  W->intags= intag;
-  W->freeme= 0;
-
-  cache->nwords++;
-
-  return(1);
-}
-
-static int bccmpwrd (UDM_BLOB_CACHE_WORD *s1, UDM_BLOB_CACHE_WORD *s2)
-{
-  int _ = strcmp(s1->word, s2->word);
-  if (! _) _ = s1->secno - s2->secno;
-  if (! _) {
-    if (s1->url_id > s2->url_id) _ = 1;
-    else if (s1->url_id < s2->url_id) _ = -1;
-    else _ = 0;
-  }
-  return(_);
-}
-
-void UdmBlobCacheSort (UDM_BLOB_CACHE *cache)
-{
-  if (cache->nwords)
-    UdmSort(cache->words, cache->nwords, sizeof(UDM_BLOB_CACHE_WORD), (udm_qsort_cmp)bccmpwrd);
-}
-
-UDM_WORD_CACHE *UdmWordCacheInit (UDM_WORD_CACHE *cache)
-{
-  if (! cache)
-  {
-    cache = UdmMalloc(sizeof(UDM_WORD_CACHE));
-    if (! cache) return(NULL);
-    cache->free = 1;
-  }
-  else
-  {
-    cache->free = 0;
-  }
-  cache->nbytes = sizeof(UDM_WORD_CACHE);
-  cache->nwords = 0;
-  cache->awords = 0;
-  cache->words = NULL;
-  cache->nurls = 0;
-  cache->aurls = 0;
-  cache->urls = NULL;
-
-  return(cache);
-}
-
-void UdmWordCacheFree (UDM_WORD_CACHE *cache)
-{
-  size_t i;
-
-  if (! cache) return;
-  for (i = 0; i < cache->nwords; i++) UDM_FREE(cache->words[i].word);
-  UDM_FREE(cache->words);
-  UDM_FREE(cache->urls);
-  cache->nbytes = sizeof(UDM_WORD_CACHE);
-  cache->nwords = 0;
-  cache->awords = 0;
-  cache->nurls = 0;
-  cache->aurls = 0;
-
-  if (cache->free) UdmFree(cache);
-}
-
-static int wccmpwrd (UDM_WORD_CACHE_WORD *s1, UDM_WORD_CACHE_WORD *s2)
-{
-  register int _;
-  if ((_= s1->seed - s2->seed))
-    return _;
-  if ((_= s1->secno - s2->secno))
-    return _;
-  if (s1->url_id > s2->url_id)
-    return 1;
-  if (s1->url_id < s2->url_id)
-    return -1;
-  if ((_= strcmp(s1->word, s2->word)))
-    return _;
-  return s1->pos - s2->pos;
-}
-
-void UdmWordCacheSort (UDM_WORD_CACHE *cache)
-{
-  UdmSort(cache->words, cache->nwords, sizeof(UDM_WORD_CACHE_WORD), (udm_qsort_cmp)wccmpwrd);
-}
-
-int UdmWordCacheAdd(UDM_WORD_CACHE *cache, urlid_t url_id, UDM_WORD *W)
-{
-  if (!W->word) return(UDM_OK);
-
-  if (cache->nwords == cache->awords)
-  {
-    UDM_WORD_CACHE_WORD *tmp;
-    tmp= UdmRealloc(cache->words, (cache->awords + 256) * sizeof(UDM_WORD_CACHE_WORD));
-    if (!tmp)
-    {
-      fprintf(stderr, "Realloc failed while adding word\n");
-      return(UDM_ERROR);
-    }
-    cache->words= tmp;
-    cache->awords+= 256;
-    cache->nbytes+= sizeof(UDM_WORD_CACHE_WORD) * 256;
-  }
-
-  if (!(cache->words[cache->nwords].word= UdmStrdup(W->word)))
-    return(UDM_ERROR);
-  cache->words[cache->nwords].url_id= url_id;
-  cache->words[cache->nwords].secno= W->secno & 0xFF;
-  cache->words[cache->nwords].pos= W->pos & 0x1FFFFF;
-  cache->words[cache->nwords].seed= UdmStrHash32(W->word) & MULTI_DICTS;
-  cache->nwords++;
-  cache->nbytes+= strlen(W->word) + 1;
-  return(UDM_OK);
-}
-
-int UdmWordCacheAddURL (UDM_WORD_CACHE *cache, urlid_t url_id)
-{
-  if (cache->nurls == cache->aurls)
-  {
-    urlid_t *tmp;
-    tmp = UdmRealloc(cache->urls, (cache->aurls + 256) * sizeof(urlid_t));
-    if (!tmp)
-    {
-      fprintf(stderr, "Realloc failed while adding word\n");
-      return(UDM_ERROR);
-    }
-    cache->urls = tmp;
-    cache->aurls += 256;
-    cache->nbytes += sizeof(urlid_t) * 256;
-  }
-
-  cache->urls[cache->nurls] = url_id;
-  cache->nurls++;
-  return(UDM_OK);
-}
