@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -49,65 +49,78 @@
 
 /**************************** DBAddr ***********************************/
 
-__C_LINK UDM_ENV * __UDMCALL UdmEnvInit(UDM_ENV *Conf){
+__C_LINK UDM_ENV * __UDMCALL
+UdmEnvInit(UDM_ENV *Conf)
+{
 
-	if(!Conf){
-		Conf=(UDM_ENV *)UdmMalloc(sizeof(UDM_ENV));
-		bzero((void*)Conf, sizeof(*Conf));
-		Conf->freeme=1;
-	}else{
-		bzero((void*)Conf, sizeof(*Conf));
-	}
-	
-	Conf->WordParam.min_word_len=1;
-	Conf->WordParam.max_word_len=32;
-	Conf->url_number=0x7FFFFFFF;
-	Conf->lcs=UdmGetCharSet("latin1");
-	Conf->bcs=UdmGetCharSet("latin1");
-	Conf->unidata= udm_unidata_default;
-	
-	return(Conf);
+  if (!Conf)
+  {
+    Conf=(UDM_ENV *)UdmMalloc(sizeof(UDM_ENV));
+    bzero((void*)Conf, sizeof(*Conf));
+    Conf->freeme=1;
+  }
+  else
+  {
+    bzero((void*)Conf, sizeof(*Conf));
+  }
+  
+  Conf->WordParam.min_word_len=1;
+  Conf->WordParam.max_word_len=32;
+  Conf->url_number=0x7FFFFFFF;
+  Conf->lcs=UdmGetCharSet("latin1");
+  Conf->bcs=UdmGetCharSet("latin1");
+  Conf->unidata= udm_unidata_default;
+  
+  return(Conf);
 }
-__C_LINK void __UDMCALL UdmEnvFree(UDM_ENV * Env){
+
+
+__C_LINK void __UDMCALL
+UdmEnvFree(UDM_ENV * Env)
+{
 #ifdef MECAB
-        mecab_destroy (Env->mecab);
+  mecab_destroy (Env->mecab);
 #endif
-	UdmRobotListFree(&Env->Robots);
-	UdmHrefListFree(&Env->Hrefs);
-	UdmResultFree(&Env->Targets);
-	UdmParserListFree(&Env->Parsers);
-	UdmStopListListFree(&Env->StopWord);
-	UdmHostListFree(&Env->Hosts);
-	
-	UdmMatchListFree(&Env->MimeTypes);
-	UdmMatchListFree(&Env->Aliases);
-	UdmMatchListFree(&Env->ReverseAliases);
-	UdmMatchListFree(&Env->Filters);
-	UdmMatchListFree(&Env->SectionFilters);
-	UdmMatchListFree(&Env->SectionMatch);
-	UdmMatchListFree(&Env->SectionHdrMatch);
-	UdmMatchListFree(&Env->SectionGsrMatch);
-		
-	UdmSynonymListListFree(&Env->Synonym);
-	UdmVarListFree(&Env->Sections);
-	UdmVarListFree(&Env->XMLEnterHooks);
-	UdmVarListFree(&Env->XMLLeaveHooks);
-	UdmVarListFree(&Env->XMLDataHooks);
-	UdmVarListFree(&Env->Cookies);
-	UdmLangMapListSave(&Env->LangMaps);
-	UdmLangMapListFree(&Env->LangMaps);
-	UdmServerListFree(&Env->Servers);
-	UdmSpellListListFree(&Env->Spells);
-	UdmAffixListListFree(&Env->Affixes);
-	UdmVarListFree(&Env->Vars);
-	UdmChineseListFree(&Env->Chi);
-	UdmChineseListFree(&Env->Thai);
-	UdmDBListFree(&Env->dbl);
-	if(Env->freeme)UDM_FREE(Env);
+  UdmRobotListFree(&Env->Robots);
+  UdmHrefListFree(&Env->Hrefs);
+  UdmResultFree(&Env->Targets);
+  UdmParserListFree(&Env->Parsers);
+  UdmStopListListFree(&Env->StopWord);
+  UdmHostListFree(&Env->Hosts);
+  
+  UdmMatchListFree(&Env->MimeTypes);
+  UdmMatchListFree(&Env->Encodings);
+  UdmMatchListFree(&Env->Aliases);
+  UdmMatchListFree(&Env->ReverseAliases);
+  UdmMatchListFree(&Env->Filters);
+  UdmMatchListFree(&Env->SectionFilters);
+  UdmMatchListFree(&Env->SectionMatch);
+  UdmMatchListFree(&Env->SectionHdrMatch);
+  UdmMatchListFree(&Env->SectionGsrMatch);
+    
+  UdmSynonymListListFree(&Env->Synonym);
+  UdmVarListFree(&Env->Sections);
+  UdmVarListFree(&Env->XMLEnterHooks);
+  UdmVarListFree(&Env->XMLLeaveHooks);
+  UdmVarListFree(&Env->XMLDataHooks);
+  UdmVarListFree(&Env->Cookies);
+  UdmLangMapListSave(&Env->LangMaps);
+  UdmLangMapListFree(&Env->LangMaps);
+  UdmServerListFree(&Env->Servers);
+  UdmSpellListListFree(&Env->Spells);
+  UdmAffixListListFree(&Env->Affixes);
+  UdmVarListFree(&Env->Vars);
+  UdmChineseListFree(&Env->Chi);
+  UdmChineseListFree(&Env->Thai);
+  UdmDBListFree(&Env->dbl);
+  if(Env->freeme)UDM_FREE(Env);
 }
 
-__C_LINK char * __UDMCALL UdmEnvErrMsg(UDM_ENV * Conf){
-  size_t	i;
+
+__C_LINK char * __UDMCALL
+UdmEnvErrMsg(UDM_ENV * Conf)
+{
+  size_t  i;
   UDM_DB *db;
 
   for(i = 0; i<Conf->dbl.nitems; i++){
@@ -126,13 +139,16 @@ __C_LINK int __UDMCALL UdmSetLockProc(UDM_ENV * Conf,
                                             int command, int type,
                                             const char *f,int l))
 {
-	Conf->LockProc=proc;
-	return(0);
+  Conf->LockProc= proc;
+  return 0;
 }
 
-int UdmEnvPrepare(UDM_ENV *Env)
+
+int
+UdmEnvPrepare(UDM_ENV *Env)
 {
 #ifdef MECAB
+   if (!Env->mecab)
    {
      char   *mecab_argv[] = { "mecab", "-F%m\" \"", "-E\" \"", "-B\" \"", NULL };
      if (!(Env->mecab= mecab_new (4, mecab_argv)))
