@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -216,11 +216,14 @@ static int CreateOrDrop(UDM_AGENT *A, enum udm_indcmd cmd)
   const char *sdir=UdmVarListFindStr(&Conf.Vars,"ShareDir",UDM_SHARE_DIR);
   UDM_DBLIST *L= &A->Conf->dbl;
   UDM_SQLMON_PARAM prm;
+  size_t num= UdmVarListFindInt(&A->Conf->Vars, "DBLimit", 0);
   
   for (i=0; i<L->nitems; i++)
   {
     FILE *infile;
     UDM_DB *db= &L->db[i];
+    if (num != 0 && num != i + 1)
+      continue;
     udm_snprintf(fname,sizeof(fname),"%s%s%s%s%s.%s.sql",
       sdir,UDMSLASHSTR, UdmDBTypeToStr(db->DBType),UDMSLASHSTR,
       UdmIndCmdStr(cmd),UdmDBModeToStr(db->DBMode));
@@ -552,7 +555,7 @@ static int usage(int level)
   FILE *file= stdout;
   fprintf(file, "\n");
   fprintf(file, "indexer from %s-%s-%s\n", PACKAGE, VERSION, UDM_DBTYPE);
-  fprintf(file, "http://www.mnogosearch.org/ (C)1998-2007, LavTech Corp.\n");
+  fprintf(file, "http://www.mnogosearch.org/ (C)1998-2009, LavTech Corp.\n");
   fprintf(file, "\n");
   fprintf(file, "Usage: indexer [OPTIONS]  [configfile]\n");
   fprintf(file, "\n");
@@ -576,7 +579,7 @@ static int usage(int level)
   fprintf(file, "  -b              block starting more than one indexer instances\n");
   fprintf(file, "  -i              insert new URLs (URLs to insert must be given using -u or -f)\n");
   fprintf(file, "  -p n            sleep n seconds after each URL\n");
-  fprintf(file, "  -w              do not warn before clearing documents from database\n");
+  fprintf(file, "  -w              do not ask for confirmation when clearing documents from the database\n");
 
 #ifdef HAVE_PTHREAD
   fprintf(file, "  -N n            run N threads\n");
@@ -593,6 +596,7 @@ static int usage(int level)
   fprintf(file, "  -L language     limit indexer to documents matching language\n");
   fprintf(file, "  -u pattern      limit indexer to documents with URLs matching pattern\n");
   fprintf(file, "                  (supports SQL LIKE wildcard '%%')\n");
+  fprintf(file, "  -D n            work with the n-th database only (i.e. with the n-th DBAddr)\n");
   fprintf(file, "  -f filename     read URLs to be indexed/inserted/cleared from file (with -a\n");
   fprintf(file, "                  or -C option, supports SQL LIKE wildcard '%%'; has no effect\n");
   fprintf(file, "                  when combined with -m option)\n");
@@ -725,7 +729,14 @@ static void UdmParseCmdLine(void)
 {
   int ch;
 
-  while ((ch = getopt(UdmARGC, UdmARGV, "QUCSIRMabheorlmqiw?-:E:F:t:u:s:n:v:L:A:D:p:N:f:c:g:y:j:d:")) != -1)
+  /* 
+    Available new options:
+    Capital letters:  B    GH JK   OP   T VXWXYZ
+    Small   letters:           k             x z
+    Digits:          123456789
+  */
+  
+  while ((ch = getopt(UdmARGC, UdmARGV, "QUCSIRMabheorlmqiw?-:E:F:t:u:s:n:v:L:A:D:p:N:f:c:g:y:j:d:D:")) != -1)
   {
     switch (ch)
     {
@@ -775,7 +786,10 @@ static void UdmParseCmdLine(void)
           UdmHrefListAdd(&Conf.Hrefs, &Href);
         }
         break;
-      case 'N': maxthreads=atoi(optarg);break;
+      case 'N':
+        maxthreads=atoi(optarg);
+        UdmVarListReplaceInt(&Conf.Vars, "CrawlerThreads", maxthreads);
+        break;
       case 'f': url_filename=optarg;break;
       case 'i': insert=1;break;
       case 'w': warnings=0;break;
@@ -783,6 +797,7 @@ static void UdmParseCmdLine(void)
       case 'd': strncpy(cname, optarg, sizeof(cname));
         cname[sizeof(cname) - 1] = '\0';
         break;
+      case 'D': UdmVarListAddStr(&Conf.Vars,"DBLimit"   , optarg); break;
       case '-':
         {
           char *arg= strchr(optarg, '=');
@@ -1225,7 +1240,7 @@ static int UdmClear(UDM_AGENT *A, const char *url_fname)
   if (warnings)
   {
     size_t i;
-    printf("You are going to delete content from database(s):\n");
+    printf("You are going to delete content from the database(s):\n");
     for (i = 0; i < Conf.dbl.nitems; i++)
     {
       const char *dbaddr;
@@ -1292,7 +1307,10 @@ UdmThreadCreate(udm_thread_t *thd, void *(*start_routine)(void*), void *arg)
 static int UdmIndex(UDM_AGENT *A)
 {
   UDM_AGENT httpd_agent;
-  size_t nbytes= maxthreads * sizeof(UDM_AGENT);
+  size_t nbytes;
+  
+  maxthreads= UdmVarListFindInt(&A->Conf->Vars, "CrawlerThreads", 1);
+  nbytes= maxthreads * sizeof(UDM_AGENT);
   ThreadIndexers= (UDM_AGENT*)UdmMalloc(nbytes);
   bzero((void*) ThreadIndexers, nbytes);
   UdmAgentInit(&httpd_agent, A->Conf, 0);
@@ -1553,6 +1571,7 @@ int main(int argc, char **argv)
      
   UdmInitMutexes();
   UdmEnvInit(&Conf);
+  UdmVarListAddEnviron(&Conf.Vars,"ENV");
   UdmEnvSetDirs(&Conf);
   UdmSetLockProc(&Conf,UdmLockProc);
   UdmSetRefProc(&Conf,UdmRefProc);
