@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -119,13 +119,7 @@ int UdmMatchExec(UDM_MATCH * Match,
   regmatch_t  subs[UDM_NSUBS];
   const char  *se;
 
-#ifdef USE_PARANOIA
-  void *paran = UdmViolationEnter();
-#endif
-  
-  /*
-  printf("'%s' '%s' '%d'\n",string,Match->pattern,Match->match_type);
-  */
+  UdmDebugEnter();
   
   switch(Match->match_type)
   {
@@ -213,8 +207,7 @@ int UdmMatchExec(UDM_MATCH * Match,
       res=0;
   }
   if (Match->nomatch) res = !res;
-  UdmViolationExit(paran);
-  return res;
+  UdmDebugReturn(res);
 }
 
 
@@ -351,6 +344,7 @@ int UdmMatchListAdd(UDM_AGENT *A, UDM_MATCHLIST *L, UDM_MATCH *M,
   N->match_type=M->match_type;
   N->case_sense=M->case_sense;
   N->nomatch=M->nomatch;
+  N->flags= M->flags;
   N->section = M->section ? (char *)UdmStrdup(M->section) : NULL;
   N->arg = M->arg ? (char*)UdmStrdup(M->arg) : NULL;
   N->arg1 = M->arg1 ? (char*)UdmStrdup(M->arg1) : NULL;
