@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -113,19 +113,16 @@ int UdmPrepareWords(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc){
   int    strip_accents;
   char   *lcsword;  /* Word in LocalCharset */
   size_t max_word_len, min_word_len, uwordlen = UDM_MAXWORDSIZE;
-#ifdef USE_PARANOIA
-  void *paran = UdmViolationEnter();
-#endif
   UDM_VAR *Sec;
   int raw_sections= (UdmVarListFindByPrefix(&Doc->Sections, "Raw.", 4) != NULL);
   const char *content_language= UdmVarListFindStr(&Doc->Sections, "Content-Language", "");
   const char *seg=  UdmVarListFindStr(&Indexer->Conf->Vars, "Segmenter", NULL);
+
+  UdmDebugEnter();
   
   if ((lcsword = (char*)UdmMalloc(12 * uwordlen + 1)) == NULL)
-  {
-    UdmViolationExit(paran);
-    return UDM_ERROR;
-  }
+    UdmDebugReturn(UDM_ERROR);
+
 
   {
     UDM_VAR *S= UdmVarListFind(&Doc->Sections,"crosswords");
@@ -305,8 +302,7 @@ int UdmPrepareWords(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc){
 
 ex:
   UDM_FREE(lcsword);
-  UdmViolationExit(paran);
-  return res;
+  UdmDebugReturn(res);
 }
 
 
@@ -733,16 +729,10 @@ int UdmHTMLParseTag(UDM_HTMLTOK * tag,UDM_DOCUMENT * Doc)
   const char *nameptr= tag->toks[0].name;
   size_t namelen= tag->toks[0].nlen;
 
-#ifdef USE_PARANOIA
-  void *paran = UdmViolationEnter();
-#endif
+  UdmDebugEnter();
 
   if(!tag->ntoks || !nameptr)
-  {
-    UdmViolationExit(paran);
-    return(0);
-  }
-
+    goto ex;
     
   for(i=0;i<tag->ntoks;i++)
   {
@@ -852,7 +842,11 @@ int UdmHTMLParseTag(UDM_HTMLTOK * tag,UDM_DOCUMENT * Doc)
     if((!tag->comment) && (Sec=UdmVarListFind(&Doc->Sections,secname)) && 
        Doc->Spider.index)
     {
-      UdmSGMLUnescape(metacont);
+      /*
+        We can't call UdmSGMLUnescape(metacont); here,
+        because it works only with dcs=lati1.
+        Un-SGML will happen later, in Unicode format.
+      */
       Item.str=metacont;
       Item.section=Sec->section;
       Item.section_name=secname;
@@ -995,9 +989,10 @@ int UdmHTMLParseTag(UDM_HTMLTOK * tag,UDM_DOCUMENT * Doc)
   UDM_FREE(metacont);
   UDM_FREE(href);
   UDM_FREE(lang);
+
+ex:
   
-  UdmViolationExit(paran);
-  return 0;
+  UdmDebugReturn(0);
 }
 
 
@@ -1031,9 +1026,8 @@ int UdmHTMLParse(UDM_AGENT *Indexer,UDM_DOCUMENT *Doc){
     We cannot use BSec and TSec later, as they are
     modified by UdmVarListReplace from UdmHTMLParseTag
   */
-#ifdef USE_PARANOIA
-  void *paran = UdmViolationEnter();
-#endif
+
+  UdmDebugEnter();
   
   bzero((void*)&Item, sizeof(Item));
   UdmHTMLTOKInit(&tag);
@@ -1129,6 +1123,5 @@ int UdmHTMLParse(UDM_AGENT *Indexer,UDM_DOCUMENT *Doc){
   }
   
   UDM_FREE(tag.lasthref);  
-  UdmViolationExit(paran);
-  return UDM_OK;
+  UdmDebugReturn(UDM_OK);
 }
