@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -83,6 +83,9 @@
 #include "udm_db.h"
 #include "udm_match.h"
 #include "udm_agent.h"
+#include "udm_contentencoding.h"
+#include "udm_url.h"
+#include "udm_indexer.h" /* For UdmDocProcessContentResponseHeaders */
 
 /************** connect with timeout **********/
 
@@ -1886,3 +1889,44 @@ int UdmGetURL(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc)
   }
   return UDM_OK;
 }
+
+/*
+  UdmDocInit() must be called of Doc before calling this function
+*/
+int
+UdmGetURLSimple(UDM_AGENT *Agent, UDM_DOCUMENT *Doc, const char *vurl)
+{
+  UDM_VARLIST *vars= &Agent->Conf->Vars;
+  size_t max_doc_size= (size_t)UdmVarListFindInt(vars,"MaxDocSize",UDM_MAXDOCSIZE);
+  if(!Doc->Buf.buf) Doc->Buf.buf = (char*)UdmMalloc(max_doc_size);
+  Doc->Buf.maxsize= max_doc_size;
+
+  UdmURLParse(&Doc->CurURL,vurl);
+  UdmVarListReplaceStr(&Doc->RequestHeaders, "Host", UDM_NULL2EMPTY(Doc->CurURL.hostname));
+  Doc->connp.hostname= (char*)UdmStrdup(UDM_NULL2EMPTY(Doc->CurURL.hostname));
+  Doc->connp.port= Doc->CurURL.port ? Doc->CurURL.port : Doc->CurURL.default_port;
+    
+  if(UdmHostLookup(&Agent->Conf->Hosts, &Doc->connp))
+  {
+  }
+
+  if  (UdmGetURL(Agent, Doc) != UDM_OK)
+    return UDM_ERROR;
+
+  UdmParseHTTPResponse(Agent, Doc);
+  UdmDocProcessContentResponseHeaders(Agent, Doc);
+
+  if(Doc->Buf.content)
+  {
+    const char *ce=UdmVarListFindStr(&Doc->Sections,"Content-Encoding","");
+#ifdef HAVE_ZLIB
+    if(!strcasecmp(ce,"gzip") || !strcasecmp(ce,"x-gzip"))
+      UdmDocUnGzip(Doc);
+    else if(!strcasecmp(ce,"deflate"))
+      UdmDocInflate(Doc);
+    else if(!strcasecmp(ce,"compress") || !strcasecmp(ce,"x-compress"))
+      UdmDocUncompress(Doc);
+#endif
+  }
+  return UDM_OK;
+}
