@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -371,7 +371,7 @@ static int ResFromXMLLeave(UDM_XML_PARSER *parser, const char *name, size_t l)
     D->Res->URLData.Item= (UDM_URLDATA*) UdmRealloc(D->Res->URLData.Item, nbytes);
     bzero((void*)&D->Res->URLData.Item[D->Res->num_rows-1], sizeof(UDM_URLDATA));
     D->Res->URLData.Item[D->Res->num_rows-1].url_id= D->Res->URLData.nitems-1;
-    D->Res->URLData.Item[D->Res->num_rows-1].coord= D->score;
+    D->Res->URLData.Item[D->Res->num_rows-1].score= D->score;
     D->Res->URLData.Item[D->Res->num_rows-1].per_site= D->per_site;
     D->Res->URLData.Item[D->Res->num_rows-1].site_id= D->site_id;
     
