@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -67,174 +67,6 @@ cmp_pos(UDM_CRD *s1, UDM_CRD *s2)
 }
 
 
-static int
-cmppattern(UDM_URLDATALIST *Data, UDM_URLDATA *D,
-           long j, const char *pattern)
-{
-  int rc;
-
-  for(; *pattern != '\0'; pattern++)
-  {
-    switch(*pattern)
-    {
-      case 'R':
-      case 'r':
-        if (D->coord > Data->Item[j].coord) return (*pattern == 'R') ? 1 : -1;
-        if (D->coord < Data->Item[j].coord) return (*pattern == 'R') ? -1 : 1;
-        break;
-      case 'P':
-      case 'p':
-        if (D->pop_rank > Data->Item[j].pop_rank) return (*pattern == 'P') ? 1 : -1;
-        if (D->pop_rank < Data->Item[j].pop_rank) return (*pattern == 'P') ? -1 : 1;
-        break;
-      case 'D':
-      case 'd':
-        if (D->last_mod_time > Data->Item[j].last_mod_time) return (*pattern == 'D') ? 1 : -1;
-        if (D->last_mod_time < Data->Item[j].last_mod_time) return (*pattern == 'D') ? -1 : 1;
-        break;
-      case 'U':
-      case 'u':
-        rc= strcmp(UDM_NULL2EMPTY(D->url), UDM_NULL2EMPTY(Data->Item[j].url));
-        if (rc) return(*pattern == 'U' ? -rc : rc);
-        break;
-      case 'S':
-      case 's':
-        rc= strcmp(UDM_NULL2EMPTY(D->section),
-                   UDM_NULL2EMPTY(Data->Item[j].section));
-        if (rc) return(*pattern == 'S' ? -rc : rc);
-        break;
-    }
-  }
-  return 0;
-}
-
-
-static int
-cmppatternRP(UDM_URLDATA *D1, UDM_URLDATA *D2)
-{
-  if (D1->coord > D2->coord) return -1;
-  if (D1->coord < D2->coord) return  1;
-  
-  if (D1->pop_rank > D2->pop_rank) return -1;
-  if (D1->pop_rank < D2->pop_rank) return 1;
-  return 0;
-}
-
-
-static int
-cmppatternR(UDM_URLDATA *D1, UDM_URLDATA *D2)
-{
-  if (D1->coord > D2->coord) return -1;
-  if (D1->coord < D2->coord) return  1;
-  return 0;
-}
-
-
-
-/****************************************************/
-
-static int
-cmpsiteid2(UDM_URLDATA *p1, UDM_URLDATA *p2)
-{
-  if (p1->site_id  > p2->site_id)  return -1;
-  if (p1->site_id  < p2->site_id)  return 1;
-#if 0
-  if (p1->coord    > p2->coord)    return -1;
-  if (p1->coord    < p2->coord)    return 1;
-  if (p1->pop_rank > p2->pop_rank) return -1;
-  if (p1->pop_rank < p2->pop_rank) return 1;
-#endif
-  return 0;
-}
-
-
-void
-UdmURLDataSortBySite(UDM_URLDATALIST *L)
-{
-  UdmSort((void*) L->Item, L->nitems, sizeof(*L->Item), (udm_qsort_cmp) cmpsiteid2);
-#if 0
-  {
-    size_t i;
-    printf("L->num: %d num: %d\n", L->ncoords, num);
-    for (i= 0; i < num; i++)
-      printf("%d %d %d %d %.6f\n", i,
-      L->Data[i].url_id, L->Data[i].site_id, L->Data[i].coord, L->Data[i].pop_rank);
-  }
-#endif
-}
-
-static void
-UdmURLDataSortByPatternRP(UDM_URLDATALIST *L)
-{
-  UdmSort((void*) L->Item, L->nitems, sizeof(*L->Item), (udm_qsort_cmp) cmppatternRP);
-}
-
-
-static void
-UdmURLDataSortByPatternR(UDM_URLDATALIST *L)
-{
-  UdmSort((void*) L->Item, L->nitems, sizeof(*L->Item), (udm_qsort_cmp) cmppatternR);
-}
-
-
-static size_t UdmH[] = {1, 5, 19, 41, 109, 209, 505, 929, 2161,
-                        3905, 8929, 16001, 36289, 64769};
-void UdmURLDataSortByPattern(UDM_URLDATALIST *L, const char *pattern)
-{
-  register ssize_t h, i, j;
-  int s = 13;
-  size_t num= L->nitems;
-  UDM_URLDATA Dat;
-
-  if (!strcmp(pattern, "RP"))
-  {
-    UdmURLDataSortByPatternRP(L);
-    goto ret;
-  }
-  if (!strcmp(pattern, "R"))
-  {
-    UdmURLDataSortByPatternR(L);
-    goto ret;
-  }
-
-  while( (s > 0) && ((num / 3) < UdmH[s])) s--;
-  while(s >= 0)
-  {
-    h= UdmH[s];
-    for (j= h; j < (ssize_t) num; j++)
-    {
-      Dat= L->Item[j];
-      i= j - h;
-D4:
-      if (cmppattern(L, &Dat, i, pattern) <= 0) goto D6;
-      L->Item[i + h]= L->Item[i];
-      i-= h;
-      if (i >= 0) goto D4;
-D6:
-      L->Item[i + h]= Dat;
-    }
-    s--;
-  }
-
-ret:
-
-#if 0
-  {
-    size_t i;
-    printf("L->num: %d num:%d pattern:%s\n", L->ncoords, num, pattern);
-    for (i= 0; i < num; i++)
-    {
-      printf("[%d]%d %d %d %d %.6f\n", i,
-             L->Coords[i].url_id, L->Coords[i].coord,
-             L->Data[i].url_id, L->Data[i].coord, (float) L->Data[i].pop_rank);
-    }
-  }
-#endif
-  return;
-}
-
-
-
 static int
 cmp_score(UDM_URL_SCORE *c1, UDM_URL_SCORE *c2)
 {
@@ -375,7 +207,8 @@ static void
 UdmDebugScore(char *str, size_t nbytes,
               UDM_SCORE_PARAM *score_param,
               UDM_SCORE_PARTS *score_parts,
-              urlid_t url_id)
+              urlid_t url_id,
+              uint4 score)
 {
   size_t dstsum= score_parts->dstnum ?
                  score_parts->dstsum *
@@ -385,11 +218,12 @@ UdmDebugScore(char *str, size_t nbytes,
   udm_snprintf(str, nbytes,
                "url_id=%d RDsum=%d distance=%d (%d/%d) "
                "minmax=%.8f density=%.8f "
-               "numword=%.8f wordform=%.8f",
+               "numword=%.8f wordform=%.8f score=%d",
                url_id, score_parts->RDsum, dstsum,
                score_parts->dstsum, score_parts->dstnum,
                score_parts->min_max_factor, score_parts->density_factor,
-               score_parts->numword_factor, score_parts->wordform_factor);
+               score_parts->numword_factor, score_parts->wordform_factor,
+               (int) score);
 }
 
 
@@ -438,7 +272,7 @@ CalcAverageWordDistanceTwoCoords(size_t wf2_secno,
 */
 static void
 CalcAverageWordDistance(size_t wf2_secno,
-                        UDM_CRD *phr, size_t num,
+                        UDM_CRD *phr, size_t num, size_t nuniq,
                         size_t *psum, size_t *pnp)
 {
   UDM_CRD *last, *last1, *phr0, *phr2;
@@ -499,6 +333,35 @@ CalcAverageWordDistance(size_t wf2_secno,
           /* "w1 W2 w3" - add two distances */
           sum+= (diff1 + diff2);
           np+= 2;
+          if (diff1 + diff2 <= 2)
+          {
+            UDM_CRD *phr3= phr1 + 1;
+            /*
+             We have three different words consequently.
+             This is much better than just pairs.
+            */
+            np+=2;
+            if (nuniq == 3)
+              np+= 512; /* Exact phrase */
+            if (phr3 < last &&
+                phr3->order != phr1->order &&
+                phr3->pos - phr1->pos <= 1)
+            {
+              UDM_CRD *phr4= phr3 + 1;
+              /* Four different words consequently */
+              np+= 2;
+              if (nuniq == 4) /* Exact phrase - all four words from the query */
+                np+= 512;
+              
+              if (phr4 < last &&
+                  phr4->order != phr3->order &&
+                  phr4->pos - phr3->pos <= 1)
+              {
+                /* Exact phrase - all five words consequently */
+                np+= 512;
+              }
+            }
+          }
         }
       }
       else
@@ -1374,7 +1237,8 @@ UdmGroupByURLLoop2(UDM_AGENT *A,
       {
         UDM_CRDLIST *L= &MergedSections[i];
         CalcAverageWordDistance(wf2[L->secno],
-                                L->Coords, L->ncoords, &dstsum, &dstnum);
+                                L->Coords, L->ncoords, param->nuniq,
+                                &dstsum, &dstnum);
       }
       score_parts.dstsum= dstsum;
       score_parts.dstnum= dstnum;
@@ -1383,18 +1247,19 @@ UdmGroupByURLLoop2(UDM_AGENT *A,
     score_parts.numword_factor= NUMWORD_FACTOR(param->numword_factor, ncoords_for_malloc);
 
     CrdTo->score= UdmCalcScore(param, From, nsections, &score_parts);
+
+    if (debug_url_id == url_id)
+    {
+      char str[255];
+      UdmDebugScore(str, sizeof(str), param, &score_parts, url_id, CrdTo->score);
+      UdmVarListAddStr(&A->Conf->Vars, "DebugScore", str);
+    }
     
     if (CrdTo->score)
     {
       CrdTo->url_id= url_id;
       CrdTo++;
     }
-    if (debug_url_id == url_id)
-    {
-      char str[255];
-      UdmDebugScore(str, sizeof(str), param, &score_parts, url_id);
-      UdmVarListAddStr(&A->Conf->Vars, "DebugScore", str);
-    }
   }
   UDM_FREE(CoordList.Coords);
   ScoreList->nitems= CrdTo - ScoreList->Item;
@@ -1763,3 +1628,146 @@ void UdmGroupByURL2(UDM_AGENT *query,
   UdmFree(prm);
 }
 
+/*************** UserScore and UserSiteScore functions ******************/
+
+/*
+  TODO:
+  reuse the same function from
+  urlidlist.c, sql.c, dbmode-blob.c, dbmode-multi, dbmode-rawblob, score.c
+*/
+static int
+cmpaurls (urlid_t *s1, urlid_t *s2)
+{
+  if (*s1 > *s2) return(1);
+  if (*s1 < *s2) return(-1);
+  return(0);
+}
+
+static void
+UdmUserScoreFindMinMax(UDM_URL_INT4_LIST *List, int *minval, int *maxval)
+{
+  size_t i;
+  *minval= *maxval= 0;
+  for (i= 0; i < List->nitems; i++)
+  {
+    UDM_URL_INT4 *Item= &List->Item[i];
+    if (*minval > Item->param)
+      *minval= Item->param;
+    if (*maxval < Item->param)
+      *maxval= Item->param;
+  }
+}
+
+
+int
+UdmUserScoreListApplyToURLScoreList(UDM_URLSCORELIST *List,
+                                    UDM_URL_INT4_LIST *UserScoreList,
+                                    int UserScoreFactor)
+{
+  size_t i;
+  int4 minval= -1;
+  int4 maxval= 1;
+  UDM_URL_SCORE *Coords= List->Item;
+
+  UdmUserScoreFindMinMax(UserScoreList, &minval, &maxval);
+  
+  for (i= 0; i < List->nitems; i++)
+  {
+    urlid_t url_id= Coords[i].url_id;
+    uint4 coord= Coords[i].score;
+    UDM_URL_INT4 *found;
+    found= (UDM_URL_INT4*) UdmBSearch(&url_id,
+                                      UserScoreList->Item,
+                                      UserScoreList->nitems,
+                                      sizeof(UDM_URL_INT4),
+                                      (udm_qsort_cmp)cmpaurls);
+    if (found)
+    {
+      if (found->param >= 0)
+        coord= coord + 
+              ((int4) (((float) (100000 - coord)) * found->param / maxval)) *
+               UserScoreFactor / 255;
+      else
+        coord= coord -
+               ((int4) (((float) coord) * found->param / minval)) *
+               UserScoreFactor / 255;
+    }
+    Coords[i].score= coord;
+  }
+  return UDM_OK;
+}
+
+
+int
+UdmUserScoreListApplyToURLDataList(UDM_URLDATALIST *List,
+                                   UDM_URL_INT4_LIST *UserScoreList,
+                                   int UserScoreFactor)
+{
+  size_t i;
+  int4 minval= -1;
+  int4 maxval= 1;
+  UDM_URLDATA *Coords= List->Item;
+
+  UdmUserScoreFindMinMax(UserScoreList, &minval, &maxval);
+
+  for (i= 0; i < List->nitems; i++)
+  {
+    urlid_t url_id= Coords[i].url_id;
+    uint4 score= Coords[i].score;
+    UDM_URL_INT4 *found;
+    found= (UDM_URL_INT4*) UdmBSearch(&url_id,
+                                      UserScoreList->Item,
+                                      UserScoreList->nitems,
+                                      sizeof(UDM_URL_INT4),
+                                      (udm_qsort_cmp)cmpaurls);
+    if (found)
+    {
+      if (found->param >= 0)
+        score= score + 
+              ((int4) (((float) (100000 - score)) * found->param / maxval)) *
+               UserScoreFactor / 255;
+      else
+        score= score -
+               ((int4) (((float) score) * found->param / minval)) *
+               UserScoreFactor / 255;
+    }
+    Coords[i].score= score;
+  }
+  return UDM_OK;
+}
+
+
+/************** DateFactor and RelevancyFactor **********/
+
+int
+UdmURLDataListApplyRelevancyFactors(UDM_AGENT *Agent,
+                                    UDM_URLDATALIST *DataList,
+                                    int RelevancyFactor,
+                                    int DateFactor)
+{
+  int i, sum;
+  time_t current_time;
+  unsigned long ticks;
+  UdmLog(Agent, UDM_LOG_DEBUG, "Start applying relevancy factors");
+  ticks= UdmStartTimer();
+  if (!(current_time= UdmVarListFindInt(&Agent->Conf->Vars, "CurrentTime", 0)))
+    time(&current_time);
+  sum= RelevancyFactor + DateFactor;
+  sum= sum ? sum : 1;
+  
+  for (i= 0; i < DataList->nitems; i++)
+  {
+    time_t doc_time= DataList->Item[i].last_mod_time;
+    uint4 *score= &DataList->Item[i].score;
+    float rel= *score * RelevancyFactor;
+    float dat= ((doc_time < current_time) ? 
+                ((float) doc_time / current_time) :
+                ((float) current_time / doc_time)) * DateFactor * 100000;
+    /* 100000 = 100% * 1000 = scale in db.c */
+    *score= (rel + dat) / sum;
+  }
+  ticks= UdmStartTimer() - ticks;
+  UdmLog(Agent, UDM_LOG_DEBUG, "Stop applying relevancy factors\t\t%.2f",
+         (float)ticks / 1000);
+  return UDM_OK;
+}
