@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -82,7 +82,7 @@ int __UDMCALL UdmResAddDocInfoSearchd(UDM_AGENT *A,
   for (i= 0; i < Res->num_rows; i++)
   {
     UDM_URLDATA *Data= &Res->URLData.Item[i + Res->first];
-    size_t Doc_dbnum= UDM_COORD2DBNUM(Data->coord);
+    size_t Doc_dbnum= UDM_COORD2DBNUM(Data->score);
 
     if (Doc_dbnum == dbnum)
     {
