@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -178,10 +178,13 @@ StoreDocCGI(UDM_AGENT *Agent, UDM_ENV *Env, UDM_VARLIST *tmpl, int httpd)
   UdmVarListReplaceStr(&Doc->Sections, "dbnum", UdmVarListFindStr(&Env->Vars, "dbnum", "0"));
   if ((qstring= UdmVarListFindStr(&Env->Vars, "ENV.QUERY_STRING", NULL)))
   {
-    /* Remove dbnum from query string */
+    /*
+      Remove dbnum from query string.
+      Note: dbnum is passed when a cached copy is displayed in cluster.
+    */
     if (!strncmp(qstring, "dbnum=", 6))
     {
-      char tmp[256];
+      char tmp[1024];
       for (qstring+= 6 ; (qstring[0] >= '0' && qstring[0] <= '9') || qstring[0]=='&' ; qstring++);
       udm_snprintf(tmp, sizeof(tmp), "%s", qstring);
       UdmVarListReplaceStr(&Doc->Sections, "ENV.QUERY_STRING", tmp);
@@ -381,7 +384,11 @@ UdmDetectSelfTemplateQueryCGI(UDM_ENV *Env,
   
   
   /* Detect template name */
-  if((env= getenv("UDMSEARCH_TEMPLATE")))
+  if (q->template_name[0])
+  {
+    /* Do nothing - already set in command line via -d option */
+  }
+  else if((env= getenv("UDMSEARCH_TEMPLATE")))
   {
     char ename[1024];
     strncpy(ename,env,sizeof(ename)-1);
@@ -509,13 +516,16 @@ int main(int argc, char ** argv)
   q.self[0]= 0;
   q.query_string[0]= 0;
 
-  while ((ch= getopt(argc, argv, "xh?")) != -1)
+  while ((ch= getopt(argc, argv, "d:xh?")) != -1)
   {
     switch (ch)
     {
       case 'x':
         httpd|= UDM_HTTPD_DAEMON;
         break;
+      case 'd':
+        udm_snprintf(q.template_name, sizeof(q.template_name), "%s", optarg);
+        break;
       case '?':
       case 'h':
       default:
@@ -824,7 +834,6 @@ int main(int argc, char ** argv)
     }
     UDM_FREE(C.Category);
     
-    stored_href= UdmVarListFindStr(&Doc->Sections, "stored_href", NULL);
     if ((!(stored_href= UdmVarListFindStr(&Doc->Sections, "stored_href", NULL)) ||
          !stored_href[0]) &&
          UdmVarListFindStr(&Doc->Sections, "CachedCopy", NULL))
