@@ -395,6 +395,11 @@ UdmSpellListListFree(UDM_SPELLLISTLIST *L)
     stUdmSpellListFree(&L->Item[i]);
   if (L->Item)
     UdmFree(L->Item);
+  /*
+    For PHP extension - it can call UdmSpellListListFree()
+    multiple times through udm_free_ispell_data()
+  */
+  UdmSpellListListInit(L);
 }
 
 int
@@ -611,6 +616,11 @@ UdmAffixListListFree(UDM_AFFIXLISTLIST *L)
   for (i=0 ; i < L->nitems; i++)
     stUdmAffixListFree(&L->Item[i]);
   UDM_FREE(L->Item);
+  /*
+    For PHP extension - it can call UdmSpellListListFree()
+    multiple times through udm_free_ispell_data()
+  */
+  UdmAffixListListInit(L);
 }
 
 int
@@ -706,7 +716,11 @@ UdmSpellListFindFirst(UDM_SPELLLIST *Sl, UDM_SPELL *Find, UDM_SPSEARCH_INFO *Si)
   UDM_SPELL *Beg= (UDM_SPELL*) UdmBSearch((const void *) Find,
                                           (const void *) Sl->Item, Sl->nitems,
                                            sizeof(Sl->Item[0]), cmpspell);
-  for ( ; Beg && Beg > Sl->Item && !strcmp(Find->word, Beg[-1].word); Beg--);
+  if (Beg)
+  {
+    for ( ; Beg > Sl->Item && !strcmp(Find->word, Beg[-1].word); Beg--);
+    Si->pos= Beg - Sl->Item;
+  }
   return Beg;
 }
 
