@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,7 +46,8 @@ typedef struct
   char  str[1];
 } UDM_IBASE_VARY;
 
-#define UDM_IBASE_MAX_BIND_PARAM 64
+#define UDM_IBASE_MAX_BIND_PARAM UDM_SQL_MAX_BIND_PARAM
+
 typedef struct
 {
   isc_db_handle DBH;        /* database handle    */
@@ -60,12 +61,28 @@ typedef struct
   size_t long_data_lengths[UDM_IBASE_MAX_BIND_PARAM]; /* For data >64K */
 } UDM_IB;
 
-static void UdmIBaseDisplayError(UDM_DB *db)
+
+static int
+UdmIBStmtFree(UDM_DB *db)
+{
+  UDM_IB *ib= (UDM_IB*) db->specific;
+  if (isc_dsql_free_statement(ib->status, &ib->query_handle, DSQL_drop))
+  {
+    db->errcode= 1;
+    return UDM_ERROR;
+  }
+  return UDM_OK;
+}
+
+
+static void
+UdmIBaseDisplayError(UDM_DB *db, const char *fname)
 {
   char *s= db->errstr;
   UDM_IB *ib= (UDM_IB*) db->specific;
   ISC_STATUS *ibstatus= ib->status;
-  
+
+  s+= sprintf(s, "%s: ", fname);
   while(isc_interprete(s ,&ibstatus))
   {
     strcat(s," ");
@@ -73,7 +90,8 @@ static void UdmIBaseDisplayError(UDM_DB *db)
   }
 }
 
-static int UdmIBaseInitDB(UDM_DB *db)
+static int
+UdmIBaseConnect(UDM_DB *db)
 {
   char dpb_buffer[256], *dpb, *e;
   const char *p;
@@ -142,13 +160,29 @@ static int UdmIBaseInitDB(UDM_DB *db)
                          &(ib->DBH), dpb_length, dpb_buffer))
   {
     db->errcode=1;
-    return(1);
+    return UDM_ERROR;
   }
-  return(0);
+  
+  /*
+    http://www.firebirdfaq.org/faq223/
+    To obtain the server version, use isc_info_svc_server_version() API func.
+    It does not work for Firebird Classic 1.0, so if you don't get an answer
+    you'll know it's Firebird Classic 1.0 or InterBase Classic 6.0.
+    Otherwise it returns a string like this:
+      LI-V2.0.0.12748 Firebird 2.0
+    or...
+      LI-V1.5.3.4870 Firebird 1.5
+  
+    If you use Firebird 2.1 (and higher), you can also use:
+      SELECT rdb$get_context('SYSTEM', 'ENGINE_VERSION')
+      from rdb$database;   
+  */
+  return UDM_OK;
 }
 
 
-static void UdmIBaseClose(UDM_DB*db)
+static void
+UdmIBaseClose(UDM_DB*db)
 {
   UDM_IB *ib= (UDM_IB*) db->specific;
   if(db->connected)
@@ -180,6 +214,7 @@ UdmIBPrepare(UDM_DB *db, const char *query)
                               &(ib->DBH), 0, NULL))
     {
       db->errcode=1;
+      UdmIBaseDisplayError(db, "isc_start_transaction");
       return UDM_ERROR;
     }
     ib->autocommit=1;
@@ -193,12 +228,13 @@ UdmIBPrepare(UDM_DB *db, const char *query)
   if (isc_dsql_allocate_statement(ib->status, &ib->DBH, &ib->query_handle))
   {
     db->errcode=1; 
+    UdmIBaseDisplayError(db, "isc_dsql_allocate_statement");
     return UDM_ERROR;
   }
   if (isc_dsql_prepare(ib->status, &ib->tr_handle, &ib->query_handle,
                        0, query, 1, NULL))
   {
-    UdmIBaseDisplayError(db);
+    UdmIBaseDisplayError(db, "isc_dsql_prepare");
     db->errcode=1; 
     return UDM_ERROR;
   }
@@ -216,7 +252,7 @@ UdmIBPrepare(UDM_DB *db, const char *query)
 
 
 static int
-UdmIBExecInternal(UDM_DB *db, UDM_SQLRES *res)
+UdmIBExecInternal(UDM_DB *db, UDM_SQLRES *res, int free_stmt)
 {
   UDM_IB *ib= (UDM_IB*) db->specific;
   ISC_STATUS  status[20]; /* To not override db->status */
@@ -474,16 +510,17 @@ blob_close:
     if (isc_dsql_execute(ib->status, &ib->tr_handle, &ib->query_handle, 1,
                          ib->in_sqlda.sqln ? &ib->in_sqlda : NULL))
     {
-      UdmIBaseDisplayError(db);
+      UdmIBaseDisplayError(db, "isc_dsql_execute");
       ib->autocommit= 1;
       db->errcode=1;
       isc_rollback_transaction(status, &ib->tr_handle);
-      isc_dsql_free_statement(status, &ib->query_handle, DSQL_drop);
+      if (free_stmt)
+        UdmIBStmtFree(db);
       return UDM_ERROR;
     }
   }
   
-  if (isc_dsql_free_statement(ib->status, &ib->query_handle, DSQL_drop))
+  if (free_stmt && UDM_OK != UdmIBStmtFree(db))
   {
     db->errcode=1; 
     isc_rollback_transaction(status, &ib->tr_handle);
@@ -549,7 +586,7 @@ UdmIBExec(UDM_DB *db)
       size_t blob_length= ib->long_data_lengths[i];
       if (UDM_OK != UdmIBCreateBlob(ib, var->sqldata, blob_length, &blob_ids[i]))
       {
-        UdmIBaseDisplayError(db);
+        UdmIBaseDisplayError(db, "UdmIBCreateBlob");
         db->errcode= 1;
         return UDM_ERROR;
       }
@@ -558,21 +595,22 @@ UdmIBExec(UDM_DB *db)
     }
   }
   
-  return UdmIBExecInternal(db, NULL);
+  return UdmIBExecInternal(db, NULL, 0);
 }
 
 
-static int sql_ibase_query(UDM_DB *db, UDM_SQLRES *res, const char *query)
+static int
+sql_ibase_query(UDM_DB *db, UDM_SQLRES *res, const char *query)
 {
   int     rc= UDM_OK;
   UDM_IB  *ib;  
 
   if(!db->connected)
   {
-    UdmIBaseInitDB(db);
+    UdmIBaseConnect(db);
     if(db->errcode)
     {
-      UdmIBaseDisplayError(db);
+      UdmIBaseDisplayError(db, "UdmIBaseConnect");
       db->errcode=1;
       return UDM_ERROR;
     }
@@ -592,6 +630,7 @@ static int sql_ibase_query(UDM_DB *db, UDM_SQLRES *res, const char *query)
                                 &(ib->DBH), 0, NULL))
       {
         db->errcode=1;
+        UdmIBaseDisplayError(db, "isc_start_transaction");
         rc= UDM_ERROR;
       }
       ib->autocommit= 0;
@@ -613,6 +652,7 @@ static int sql_ibase_query(UDM_DB *db, UDM_SQLRES *res, const char *query)
       if (isc_commit_transaction(ib->status, &ib->tr_handle))
       {
         db->errcode=1;
+        UdmIBaseDisplayError(db, "isc_commit_transaction");
         rc= UDM_ERROR;
       }
       ib->tr_handle=NULL;
@@ -631,13 +671,15 @@ static int sql_ibase_query(UDM_DB *db, UDM_SQLRES *res, const char *query)
   if (UdmIBPrepare(db, query))
     return UDM_ERROR;
 
-  if (UdmIBExecInternal(db, res))
+  if (UdmIBExecInternal(db, res, 1))
     return UDM_ERROR;
   
   return rc;
 }
 
-static int UdmIBaseQuery(UDM_DB *db, UDM_SQLRES *res, const char *query)
+
+static int
+UdmIBaseQuery(UDM_DB *db, UDM_SQLRES *res, const char *query)
 {
   int rc;
   db->errcode= 0;
@@ -650,8 +692,6 @@ static int UdmIBaseQuery(UDM_DB *db, UDM_SQLRES *res, const char *query)
 
   if(UDM_OK != (rc= sql_ibase_query(db,res,query)))
   {
-    UdmIBaseDisplayError(db);
-    
     if(strstr(db->errstr,"uplicat") || strstr(db->errstr,"UNIQUE"))
     {
       db->errcode=0;
@@ -667,14 +707,18 @@ static int UdmIBaseQuery(UDM_DB *db, UDM_SQLRES *res, const char *query)
   return rc;
 }
 
-static int UdmIBBegin(UDM_DB *db)
+
+static int
+UdmIBBegin(UDM_DB *db)
 {
   int rc= UdmIBaseQuery(db,NULL,"BEGIN");
   db->commit_fl = 1;
   return rc;
 }
 
-static int UdmIBCommit(UDM_DB *db)
+
+static int
+UdmIBCommit(UDM_DB *db)
 {
   int rc= UdmIBaseQuery(db,NULL,"COMMIT");
   db->commit_fl = 1;
@@ -687,6 +731,7 @@ UdmSQLType2IBType(int udm_type)
 {
   switch (udm_type)
   {
+    case UDM_SQLTYPE_INT:           return SQL_LONG;
     case UDM_SQLTYPE_LONGVARBINARY: return SQL_BLOB;
   }
   return SQL_TEXT;
@@ -695,6 +740,7 @@ UdmSQLType2IBType(int udm_type)
 
 static int short flag0= 0;
 
+
 static int
 UdmIBBind(UDM_DB *db, int position, void *data, int size, int type)
 {
@@ -716,14 +762,16 @@ UdmIBBind(UDM_DB *db, int position, void *data, int size, int type)
 
 UDM_SQLDB_HANDLER udm_sqldb_ibase_handler =
 {
-  NULL,
+  UdmSQLEscStrGeneric,
   UdmIBaseQuery,
+  UdmIBaseConnect,
   UdmIBaseClose,
   UdmIBBegin,
   UdmIBCommit,
   UdmIBPrepare,
   UdmIBBind,
   UdmIBExec,
+  UdmIBStmtFree,
   UdmSQLFetchRowSimple,
   UdmSQLStoreResultSimple,
   UdmSQLFreeResultSimple,
