@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,8 +39,8 @@
 
 #include <oci.h>
 
+#define MAX_BIND_PARAM		UDM_SQL_MAX_BIND_PARAM
 #define	MAX_COLS_IN_TABLE	32
-#define MAX_BIND_PARAM		34
 #define BUF_OUT_SIZE		16
 
 /* (C) copyleft 2000 Anton Zemlyanov, az@hotmail.ru */
@@ -58,22 +58,31 @@ typedef struct udm_oracle_db_st
 } UDM_ORACLE_DB;
 
 
-#define SQL_OK(rc)  ((rc==OCI_SUCCESS)||(rc==OCI_SUCCESS_WITH_INFO))
-
-static int udm_oracle_types[] =
+static inline int
+SQL_OK(int rc)
 {
-  0,
-  SQLT_LBI,
-  SQLT_LNG
-};
+  return (rc == OCI_SUCCESS) || (rc == OCI_SUCCESS_WITH_INFO);
+}
+
 
 static int
-UdmOracle8DisplayError(UDM_DB *db, const char *fname)
+UdmOracleTypes[] =
+{
+  0,         /* UDM_SQLTYPE_UNKNOWN       */
+  SQLT_LBI,  /* UDM_SQLTYPE_LONGVARBINARY */
+  SQLT_LNG,  /* UDM_SQLTYPE_LONGVARCHAR   */
+  SQLT_CHR,  /* UDM_SQLTYPE_VARCHAR       */
+  SQLT_INT,  /* UDM_SQLTYPE_INT           */
+};
+
+
+static int
+UdmOraDisplayError(UDM_DB *db, const char *fname)
 {
   sb4  errcode=0;
   char  errbuf[512];
   char  *ptr;
-  UDM_ORACLE_DB *ob = (UDM_ORACLE_DB *)db->specific;
+  UDM_ORACLE_DB *ob= (UDM_ORACLE_DB *)db->specific;
 
   db->errstr[0]='\0';
 
@@ -95,9 +104,9 @@ UdmOracle8DisplayError(UDM_DB *db, const char *fname)
       OCIErrorGet((dvoid *)ob->errhp, (ub4) 1, (text *) NULL, &errcode,
                   (text *)errbuf, (ub4) sizeof(errbuf), OCI_HTYPE_ERROR);
       ptr=errbuf;
-      while(*ptr)
+      while (*ptr)
       {
-        if(*ptr=='\n')
+        if (*ptr=='\n')
           *ptr='!';
         ++ptr;
       }
@@ -121,25 +130,28 @@ UdmOracle8DisplayError(UDM_DB *db, const char *fname)
 }
 
 
-static void UdmOracle8Close(UDM_DB *db)
+static void
+UdmOraClose(UDM_DB *db)
 {
-  UDM_ORACLE_DB *ob = (UDM_ORACLE_DB *)db->specific;
-  if(!db->connected)
+  UDM_ORACLE_DB *ob= (UDM_ORACLE_DB *)db->specific;
+  if (!db->connected)
     return;
   db->errcode=OCILogoff(ob->svchp,ob->errhp);
   db->connected=0;
   UDM_FREE(db->specific);
 }
 
-static int UdmOracle8InitDB(UDM_DB *db)
+
+static int
+UdmOraConnect(UDM_DB *db)
 {
   const char* DBUser= UdmVarListFindStr(&db->Vars, "DBUser", "");
   const char* DBPass= UdmVarListFindStr(&db->Vars, "DBPass", "");
   UDM_ORACLE_DB *ob;
 
-  db->specific = UdmMalloc(sizeof(UDM_ORACLE_DB));
+  db->specific= UdmMalloc(sizeof(UDM_ORACLE_DB));
   if (! db->specific) return(UDM_ERROR);
-  ob = (UDM_ORACLE_DB *)db->specific;
+  ob= (UDM_ORACLE_DB *)db->specific;
   OCIInitialize( OCI_DEFAULT, NULL, NULL, NULL, NULL );
   OCIEnvInit( &(ob->envhp), OCI_DEFAULT, 0, NULL );
   OCIHandleAlloc( ob->envhp, (dvoid **)&(ob->errhp), OCI_HTYPE_ERROR,0, NULL );
@@ -147,55 +159,78 @@ static int UdmOracle8InitDB(UDM_DB *db)
                         (const text *)DBUser, strlen(DBUser), (const text *)DBPass, strlen(DBPass),
                         (const text *)db->DBName, strlen(db->DBName));
   
-  if(db->errcode!=OCI_SUCCESS)
-    return -1;
+  if (db->errcode!=OCI_SUCCESS)
+    return UDM_ERROR;
   
-  db->errcode=0;
-  db->connected=1;
-  db->commit_fl=0;
+  db->errcode= 0;
+  db->connected= 1;
+  db->commit_fl= 0;
 
   /*sql_query(((UDM_DB*)(db)), "ALTER SESSION SET SQL_TRACE=TRUE"); */
   
-  return 0;
+  return UDM_OK;
 }
 
-static int oracle_read_blob (UDM_DB *db, OCILobLocator *lobp, UDM_PSTR *items)
+
+static int
+oracle_read_blob(UDM_DB *db, OCILobLocator *lobp, UDM_PSTR *items)
 {
   int rc;
   ub4 len;
   char *buf;
-  UDM_ORACLE_DB *ob = (UDM_ORACLE_DB *)db->specific;
+  UDM_ORACLE_DB *ob= (UDM_ORACLE_DB *)db->specific;
 
-  rc = OCILobGetLength(ob->svchp, ob->errhp, lobp, &len);
-  len= len*3; /* Convert number of characters to number of bytes */
+  rc= OCILobGetLength(ob->svchp, ob->errhp, lobp, &len);
+   /*
+     Convert number of characters to number of bytes.
+     It used to work with len*3, however oracle-xe-univ-10.2.0.1
+     ("universal" edition) requires len*4.
+  */
+  len= len*4;
   if (!SQL_OK(rc))
   {
-    db->errcode = rc;
+    db->errcode= rc;
     return(UDM_ERROR);
   }
 
-  buf = UdmMalloc(len + 1);
+  buf= UdmMalloc(len + 1);
   if (! buf)
   {
     return(UDM_ERROR);
   }
 
-  rc = OCILobRead(ob->svchp, ob->errhp, lobp, &len, 1, buf, len, 0, 0, 0, SQLCS_IMPLICIT);
+  rc= OCILobRead(ob->svchp, ob->errhp, lobp, &len, 1, buf, len, 0, 0, 0, SQLCS_IMPLICIT);
   if (!SQL_OK(rc))
   {
-    db->errcode = rc;
+    db->errcode= rc;
     UdmFree(buf);
     return(UDM_ERROR);
   }
-  buf[len] = 0;
+  buf[len]= 0;
 
-  items->len = len;
-  items->val = buf;
+  items->len= len;
+  items->val= buf;
 
   return(UDM_OK);
 }
 
-static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
+
+static int
+UdmOraStmtFree(UDM_DB *db)
+{
+  int rc;
+  UDM_ORACLE_DB *ob= (UDM_ORACLE_DB *)db->specific;
+  if (!SQL_OK(rc= OCIHandleFree(ob->stmthp, OCI_HTYPE_STMT)))
+  {
+    db->errcode= rc;
+    return UDM_ERROR;
+  }
+  return UDM_OK;
+}
+
+
+static int
+sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
 {
   sword  rc;
   ub2  stmt_type;
@@ -212,34 +247,34 @@ static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
   int	col_size[MAX_COLS_IN_TABLE];
   int	col_type[MAX_COLS_IN_TABLE];
   sb2	indbuff[MAX_COLS_IN_TABLE][BUF_OUT_SIZE];
-  UDM_ORACLE_DB *ob = (UDM_ORACLE_DB *)db->specific;
+  UDM_ORACLE_DB *ob= (UDM_ORACLE_DB *)db->specific;
 
   db->errcode=0;
-  rc=OCIHandleAlloc(ob->envhp, (dvoid *)&(ob->stmthp), OCI_HTYPE_STMT,0,NULL);
-  if(!SQL_OK(rc))
+  rc= OCIHandleAlloc(ob->envhp, (dvoid *)&(ob->stmthp), OCI_HTYPE_STMT,0,NULL);
+  if (!SQL_OK(rc))
   {
     db->errcode=rc;
     return(UDM_ERROR);
   }
-  rc=OCIStmtPrepare(ob->stmthp,ob->errhp,(const text *)qbuf,strlen(qbuf),
-                    OCI_NTV_SYNTAX,OCI_DEFAULT);
-  if(!SQL_OK(rc))
+  rc= OCIStmtPrepare(ob->stmthp,ob->errhp,(const text *)qbuf,strlen(qbuf),
+                     OCI_NTV_SYNTAX,OCI_DEFAULT);
+  if (!SQL_OK(rc))
   {
     db->errcode=rc;
     return(UDM_ERROR);
   }
 
-  num_rec = 1;
+  num_rec= 1;
     
-  rc=OCIAttrGet(ob->stmthp,OCI_HTYPE_STMT,&stmt_type,0,
-                OCI_ATTR_STMT_TYPE,ob->errhp);
-  if(!SQL_OK(rc))
+  rc= OCIAttrGet(ob->stmthp,OCI_HTYPE_STMT,&stmt_type,0,
+                 OCI_ATTR_STMT_TYPE,ob->errhp);
+  if (!SQL_OK(rc))
   {
     db->errcode=rc;
     return(UDM_ERROR);
   }
   
-  if(stmt_type!=OCI_STMT_SELECT)
+  if (stmt_type!=OCI_STMT_SELECT)
   {
     /* non-select statements */
     /* COMMIT_ON_SUCCESS in inefficient */
@@ -248,38 +283,33 @@ static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
     else
       oci_fl=OCI_COMMIT_ON_SUCCESS;
 
-    rc=OCIStmtExecute(ob->svchp,ob->stmthp,ob->errhp, num_rec,0,
-                      NULL,NULL,oci_fl);
+    rc= OCIStmtExecute(ob->svchp,ob->stmthp,ob->errhp, num_rec,0,
+                       NULL,NULL,oci_fl);
     
     if (num_rec>1)
       (void) OCITransCommit(ob->svchp, ob->errhp, (ub4) 0);
     
-    if(!SQL_OK(rc))
+    if (!SQL_OK(rc))
     {
       db->errcode=rc;
       OCIErrorGet((dvoid *)ob->errhp, (ub4) 1, (text *) NULL, &errcode,
                    (text *)errbuf, (ub4) sizeof(errbuf), OCI_HTYPE_ERROR);
-      if(strncmp(errbuf,"ORA-00001",9)) /* ignore ORA-00001 */
+      if (strncmp(errbuf,"ORA-00001",9)) /* ignore ORA-00001 */
         return(UDM_ERROR);
       else 
         db->errcode=OCI_SUCCESS;
     }
     
     /* OCI_ATTR_ROW_COUNT of db->stmthp - Rows affected */
-    rc=OCIHandleFree(ob->stmthp,OCI_HTYPE_STMT);
-    if(!SQL_OK(rc))
-    {
-      db->errcode=rc;
-      return(UDM_ERROR);
-    }
-    return(UDM_OK);
+    
+    goto stmt_free;
   }
 
   /* select statements */
 
   rc=OCIStmtExecute(ob->svchp,ob->stmthp,ob->errhp,0,0,
                     NULL,NULL,OCI_DEFAULT);
-  if(!SQL_OK(rc))
+  if (!SQL_OK(rc))
   {
     db->errcode=rc;
     return(UDM_ERROR);
@@ -287,7 +317,7 @@ static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
 
   /*describe the select list and define buffers for the select list */
   colcnt=0;
-  while(OCIParamGet(ob->stmthp,OCI_HTYPE_STMT,ob->errhp,(dvoid *)&(ob->param),
+  while (OCIParamGet(ob->stmthp,OCI_HTYPE_STMT,ob->errhp,(dvoid *)&(ob->param),
                     colcnt+1 ) == OCI_SUCCESS)
   {
     
@@ -299,24 +329,24 @@ static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
     
     rc=OCIAttrGet(ob->param,OCI_DTYPE_PARAM,(dvoid*) &namep, (ub4*) &str_len,
                   (ub4) OCI_ATTR_NAME, ob->errhp);
-    if(!SQL_OK(rc))
+    if (!SQL_OK(rc))
     {
       db->errcode=rc;
       return(UDM_ERROR);
     }
     namep[str_len]= '\0';
-    result->Fields[colcnt].sqlname = (char*)UdmStrdup(namep);
+    result->Fields[colcnt].sqlname= (char*)UdmStrdup(namep);
     
     rc=OCIAttrGet(ob->param,OCI_DTYPE_PARAM,&(coltype),0,
                   OCI_ATTR_DATA_TYPE,ob->errhp);
-    if(!SQL_OK(rc))
+    if (!SQL_OK(rc))
     {
       db->errcode=rc;
       return(UDM_ERROR);
     }
     rc=OCIAttrGet(ob->param,OCI_DTYPE_PARAM,&colsize,0,
                   OCI_ATTR_DATA_SIZE,ob->errhp);
-    if(!SQL_OK(rc))
+    if (!SQL_OK(rc))
     {
       db->errcode=rc;
       return(UDM_ERROR);
@@ -325,16 +355,16 @@ static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
     /* OCIStmtFetch do not terminate data with \0 -
     add a byte for terminator to define buffers - insurance*/
 
-    col_type[colcnt] = coltype;
+    col_type[colcnt]= coltype;
     switch(coltype) {
     size_t i;
     case SQLT_BLOB:
     case SQLT_CLOB:
-      col_size[colcnt] = sizeof(OCILobLocator *);
-      defbuff[colcnt] = (char *)UdmXmalloc(sizeof(OCILobLocator *) * BUF_OUT_SIZE);
-      for (i = 0; i < BUF_OUT_SIZE; i++)
+      col_size[colcnt]= sizeof(OCILobLocator *);
+      defbuff[colcnt]= (char *)UdmXmalloc(sizeof(OCILobLocator *) * BUF_OUT_SIZE);
+      for (i= 0; i < BUF_OUT_SIZE; i++)
       {
-        rc = OCIDescriptorAlloc(ob->envhp, (dvoid **)(defbuff[colcnt] + i * sizeof(OCILobLocator *)), OCI_DTYPE_LOB, 0, 0);
+        rc= OCIDescriptorAlloc(ob->envhp, (dvoid **)(defbuff[colcnt] + i * sizeof(OCILobLocator *)), OCI_DTYPE_LOB, 0, 0);
 	if (!SQL_OK(rc))
 	{
 	  db->errcode=rc;
@@ -370,7 +400,7 @@ static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
                         col_size[colcnt],SQLT_CHR,
                         &(indbuff[colcnt]),0,0,OCI_DEFAULT);
     
-    if(!SQL_OK(rc))
+    if (!SQL_OK(rc))
     {
       db->errcode=rc;
        return(UDM_ERROR);
@@ -379,7 +409,7 @@ static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
     rc=OCIDefineArrayOfStruct(ob->defb[colcnt], ob->errhp,
                               col_size[colcnt],
                               sizeof(indbuff[0][0]), 0, 0);
-    if(!SQL_OK(rc))
+    if (!SQL_OK(rc))
     {
       db->errcode=rc;
       return(UDM_ERROR);
@@ -389,7 +419,7 @@ static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
   result->nCols=colcnt;
 
   /* Now fetch the selected rows into the memory */
-  while(1)
+  while (1)
   {
     if (db->res_limit)
       if (db->res_limit == result->nRows)
@@ -398,29 +428,29 @@ static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
       Fix me: Process of determine real fetched rows
       Fill indicator buffer for value -6 to
     */
-    for(row=0; row<BUF_OUT_SIZE; row++)
+    for (row=0; row<BUF_OUT_SIZE; row++)
       indbuff[result->nCols-1][row]=-6;
 
     rc=OCIStmtFetch(ob->stmthp,ob->errhp,BUF_OUT_SIZE,
                     OCI_FETCH_NEXT,OCI_DEFAULT);
     
-    if((rc!=OCI_NO_DATA) && !SQL_OK(rc))
+    if ((rc!=OCI_NO_DATA) && !SQL_OK(rc))
     {
       db->errcode=rc;
       return(UDM_ERROR);
     }
 
     /* Find number of fetched rows */
-    for(buf_nRows=0; buf_nRows<BUF_OUT_SIZE; buf_nRows++)
+    for (buf_nRows=0; buf_nRows<BUF_OUT_SIZE; buf_nRows++)
     {
-      if(indbuff[result->nCols-1][buf_nRows]==-6)
+      if (indbuff[result->nCols-1][buf_nRows]==-6)
       break;
     }
 
-    if(!buf_nRows)
+    if (!buf_nRows)
       break;
 
-    if(!result->nRows)  /* first row - allocate */
+    if (!result->nRows)  /* first row - allocate */
       result->Items=(UDM_PSTR *)UdmXmalloc(buf_nRows*result->nCols*sizeof(UDM_PSTR));
     else
       result->Items=(UDM_PSTR *)UdmXrealloc(result->Items,
@@ -430,14 +460,14 @@ static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
     if (db->res_limit)
       buf_nRows=result->nRows+buf_nRows<=db->res_limit?buf_nRows:(db->res_limit-result->nRows);
     
-    for(row=0; row<buf_nRows; row++)
+    for (row=0; row<buf_nRows; row++)
     {
-      for(cnt=0; cnt<result->nCols; ++cnt)
+      for (cnt=0; cnt<result->nCols; ++cnt)
       {
-        if(indbuff[cnt][row]==OCI_IND_NULL)
+        if (indbuff[cnt][row]==OCI_IND_NULL)
         {
-          result->Items[(result->nRows + row) * result->nCols + cnt].val = (char *)UdmStrdup("");
-          result->Items[(result->nRows + row) * result->nCols + cnt].len = 0;
+          result->Items[(result->nRows + row) * result->nCols + cnt].val= (char *)UdmStrdup("");
+          result->Items[(result->nRows + row) * result->nCols + cnt].len= 0;
         }
         else
         {
@@ -448,7 +478,7 @@ static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
           }
           else if (col_type[cnt] == SQLT_CLOB || col_type[cnt] == SQLT_BLOB)
 	  {
-	    OCILobLocator **lobp = (OCILobLocator **)(defbuff[cnt] + offset);
+	    OCILobLocator **lobp= (OCILobLocator **)(defbuff[cnt] + offset);
 	    oracle_read_blob(db, *lobp, &result->Items[(result->nRows + row) * result->nCols + cnt]);
           }
 	  else
@@ -456,49 +486,47 @@ static int sql_oracle_query(UDM_DB *db, UDM_SQLRES *result, const char *qbuf)
             char *val=UdmXmalloc(col_size[cnt]+1);
             udm_snprintf(val, col_size[cnt]+1, "%s",
                          defbuff[cnt]+offset);
-            result->Items[(result->nRows + row) * result->nCols + cnt].val = (char *)UdmStrdup(UdmRTrim(val, " "));
-            result->Items[(result->nRows + row) * result->nCols + cnt].len = strlen(result->Items[(result->nRows + row) * result->nCols + cnt].val);
+            result->Items[(result->nRows + row) * result->nCols + cnt].val= (char *)UdmStrdup(UdmRTrim(val, " "));
+            result->Items[(result->nRows + row) * result->nCols + cnt].len= strlen(result->Items[(result->nRows + row) * result->nCols + cnt].val);
             UDM_FREE(val);
           }
         }
       }
     }
     result->nRows+=row;
-    if(rc==OCI_NO_DATA)
+    if (rc==OCI_NO_DATA)
       break;
-    if(!SQL_OK(rc))
+    if (!SQL_OK(rc))
     {
       db->errcode=rc;
       return(UDM_ERROR);
     }
   }
   
-  for (cnt = 0; cnt < result->nCols; cnt++)
+  for (cnt= 0; cnt < result->nCols; cnt++)
   {
     if (col_type[cnt] == SQLT_CLOB || col_type[cnt] == SQLT_BLOB)
     {
       size_t i;
-      for (i = 0; i < BUF_OUT_SIZE; i++)
+      for (i= 0; i < BUF_OUT_SIZE; i++)
       {
-        OCILobLocator **lobp = (OCILobLocator **)(defbuff[cnt] + i * sizeof(OCILobLocator *));
+        OCILobLocator **lobp= (OCILobLocator **)(defbuff[cnt] + i * sizeof(OCILobLocator *));
         OCIDescriptorFree(*lobp, OCI_DTYPE_LOB);
       }
     }
     UdmFree(defbuff[cnt]);
   }
 
-  rc= OCIHandleFree(ob->stmthp,OCI_HTYPE_STMT);
-  db->res_limit = 0;
-  if(!SQL_OK(rc))
-  {
-    db->errcode=rc;
-    return(UDM_ERROR);
-  }
-  return(UDM_OK);
+stmt_free:
+
+  db->res_limit= 0;
+  rc= UdmOraStmtFree(db);
+  return rc;
 }
 
 
-static int UdmOracle8Query(UDM_DB *db, UDM_SQLRES *res, const char *qbuf)
+static int
+UdmOraQuery(UDM_DB *db, UDM_SQLRES *res, const char *qbuf)
 {
   int rc;
   
@@ -508,15 +536,15 @@ static int UdmOracle8Query(UDM_DB *db, UDM_SQLRES *res, const char *qbuf)
     res->db= db;
   }
 
-  if(!db->connected)
+  if (!db->connected)
   {
     size_t i;
     for (i= 0; i < 3; i++)
     {
-      UdmOracle8InitDB(db);
-      if(db->errcode)
+      UdmOraConnect(db);
+      if (db->errcode)
       {
-        UdmOracle8DisplayError(db, "InitDB");
+        UdmOraDisplayError(db, "InitDB");
         if (strstr(db->errstr, "ORA-12520"))
         {
           /* No listener available, try again */
@@ -525,11 +553,11 @@ static int UdmOracle8Query(UDM_DB *db, UDM_SQLRES *res, const char *qbuf)
           UDMSLEEP(1);
           continue;
         }
-        return(UDM_ERROR);
+        return UDM_ERROR;
       }
       else
       {
-        db->connected=1;
+        db->connected= 1;
         goto query;
       }
     }
@@ -542,123 +570,127 @@ query:
   rc= sql_oracle_query(db, res, qbuf);
 
 error:
-  if(db->errcode)
+  if (db->errcode)
   {
-    UdmOracle8DisplayError(db, "ExecDirect");
-    return(UDM_ERROR);
+    UdmOraDisplayError(db, "ExecDirect");
+    return UDM_ERROR;
   }
-  return(rc);
-}
-
-static int UdmOracle8Prepare (UDM_DB *db, const char *query)
-{
-  int _;
-  UDM_ORACLE_DB *ob = (UDM_ORACLE_DB *)db->specific;
-
-  db->errcode = 0;
-  db->errstr[0] = 0;
-  _ = OCIHandleAlloc(ob->envhp, (dvoid *)&(ob->stmthp), OCI_HTYPE_STMT, 0, NULL);
-  if (! SQL_OK(_))
-  {
-    db->errcode = _;
-    UdmOracle8DisplayError(db, "Prepare");
-    return(UDM_ERROR);
-  }
-
-  _ = OCIStmtPrepare(ob->stmthp, ob->errhp, (const unsigned char *)query,
-                     strlen(query), OCI_NTV_SYNTAX, OCI_DEFAULT);
-  if (! SQL_OK(_))
-  {
-    db->errcode = _;
-    UdmOracle8DisplayError(db, "Prepare");
-    return(UDM_ERROR);
-  }
-  return(UDM_OK);
-}
-
-
-static int UdmOracle8Bind (UDM_DB *db, int position,
-                           void *data, int size, int type)
-{
-  int _;
-  UDM_ORACLE_DB *ob = (UDM_ORACLE_DB *)db->specific;
-  
-  db->errcode = 0;
-  db->errstr[0] = 0;
-  ob->bndhp[position] = NULL;
-  _ = OCIBindByPos(ob->stmthp, &ob->bndhp[position], ob->errhp,
-  (ub4)position,
-  (dvoid *)data,
-  (sb4)size,
-  (ub2)udm_oracle_types[type],
-  (dvoid *)0, (ub2 *)0, (ub2 *)0, (ub4)0, (ub4 *)0, (ub4)OCI_DEFAULT);
-
-  if (! SQL_OK(_))
-  {
-    db->errcode = _;
-    UdmOracle8DisplayError(db, "Bind");
-    return(UDM_ERROR);
-  }
-  return(UDM_OK);
-}
-
-static int UdmOracle8Exec (UDM_DB *db)
-{
-  int _;
-  size_t num_rec = 1;
-  UDM_ORACLE_DB *ob = (UDM_ORACLE_DB *)db->specific;
-
-  db->errcode = 0;
-  db->errstr[0] = 0;
-  _ = OCIStmtExecute(ob->svchp, ob->stmthp, ob->errhp, num_rec, 0, NULL, NULL, OCI_DEFAULT);
-  if (! SQL_OK(_))
-  {
-    db->errcode = _;
-    UdmOracle8DisplayError(db, "Exec");
-    return(UDM_ERROR);
-  }
-  
-  _ = OCIHandleFree(ob->stmthp, OCI_HTYPE_STMT);
-  if (! SQL_OK(_))
-  {
-    db->errcode = _;
-    UdmOracle8DisplayError(db, "Exec");
-    return(UDM_ERROR);
-  }
-  return(UDM_OK);
-}
-
-
-static int UdmOraBegin(UDM_DB *db)
-{
-  int rc= UdmOracle8Query(db,NULL,"COMMIT");
-  db->commit_fl = 1;
   return rc;
 }
 
 
-static int UdmOraCommit(UDM_DB *db)
+static int
+UdmOraPrepare(UDM_DB *db, const char *query)
 {
-  int rc= UdmOracle8Query(db,NULL,"COMMIT");
-  db->commit_fl = 0;
+  int rc;
+  UDM_ORACLE_DB *ob= (UDM_ORACLE_DB *)db->specific;
+
+  db->errcode= 0;
+  db->errstr[0]= 0;
+  rc= OCIHandleAlloc(ob->envhp, (dvoid *)&(ob->stmthp), OCI_HTYPE_STMT, 0, NULL);
+  if (!SQL_OK(rc))
+  {
+    db->errcode= rc;
+    UdmOraDisplayError(db, "Prepare");
+    return UDM_ERROR;
+  }
+
+  rc= OCIStmtPrepare(ob->stmthp, ob->errhp, (const unsigned char *)query,
+                     strlen(query), OCI_NTV_SYNTAX, OCI_DEFAULT);
+  if (!SQL_OK(rc))
+  {
+    db->errcode= rc;
+    UdmOraDisplayError(db, "Prepare");
+    return UDM_ERROR;
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmOraBind(UDM_DB *db, int position, void *data, int size, int type)
+{
+  int rc;
+  UDM_ORACLE_DB *ob= (UDM_ORACLE_DB *)db->specific;
+  
+  db->errcode= 0;
+  db->errstr[0]= 0;
+  ob->bndhp[position]= NULL;
+  rc= OCIBindByPos(ob->stmthp, &ob->bndhp[position], ob->errhp,
+                  (ub4) position,
+                  (dvoid *) data,
+                  (sb4) size,
+                  (ub2) UdmOracleTypes[type],
+                  (dvoid *) 0,
+                  (ub2 *)0, (ub2 *)0, (ub4)0, (ub4 *)0,
+                  (ub4) OCI_DEFAULT);
+
+  if (!SQL_OK(rc))
+  {
+    db->errcode= rc;
+    UdmOraDisplayError(db, "Bind");
+    return UDM_ERROR;
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmOraExec(UDM_DB *db)
+{
+  int rc;
+  size_t num_rec= 1;
+  UDM_ORACLE_DB *ob= (UDM_ORACLE_DB *)db->specific;
+  
+  db->errcode= 0;
+  db->errstr[0]= 0;
+  
+  if (!SQL_OK(rc= OCIStmtExecute(ob->svchp, ob->stmthp, ob->errhp,
+                                 num_rec, 0, NULL, NULL, OCI_DEFAULT)))
+  {
+    db->errcode= rc;
+    UdmOraDisplayError(db, "Exec");
+    return UDM_ERROR;
+  }
+
+  return UDM_OK;
+}
+
+
+static int
+UdmOraBegin(UDM_DB *db)
+{
+  int rc= UdmOraQuery(db,NULL,"COMMIT");
+  db->commit_fl= 1;
+  return rc;
+}
+
+
+static int
+UdmOraCommit(UDM_DB *db)
+{
+  int rc= UdmOraQuery(db,NULL,"COMMIT");
+  db->commit_fl= 0;
   return rc;
 }
 
 
 UDM_SQLDB_HANDLER udm_sqldb_oracle_handler =
 {
-  NULL,
-  UdmOracle8Query,
-  UdmOracle8Close,
+  UdmSQLEscStrGeneric,
+  UdmOraQuery,
+  UdmOraConnect,
+  UdmOraClose,
   UdmOraBegin,
   UdmOraCommit,
-  UdmOracle8Prepare,
-  UdmOracle8Bind,
-  UdmOracle8Exec,
+  UdmOraPrepare,
+  UdmOraBind,
+  UdmOraExec,
+  UdmOraStmtFree,
   UdmSQLFetchRowSimple,
   UdmSQLStoreResultSimple,
   UdmSQLFreeResultSimple,
-  UdmOracle8Query
+  UdmOraQuery
 };
 
 #endif
