@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -63,7 +63,8 @@ UDM_STOPWORD
 }
 */
 
-static int cmpstop(const void *s1,const void *s2)
+static int
+cmpstop(const void *s1,const void *s2)
 {
   return(strcmp(((const UDM_STOPWORD*)s1)->word,((const UDM_STOPWORD*)s2)->word));
 }
@@ -76,12 +77,16 @@ UdmStopListInit(UDM_STOPLIST *List)
 }
 
 
-void UdmStopListSort(UDM_STOPLIST *List){
+void
+UdmStopListSort(UDM_STOPLIST *List)
+{
   /* Sort stoplist to run binary search later */
   UdmSort(List->StopWord,List->nstopwords,sizeof(UDM_STOPWORD),cmpstop);
 }
 
-int UdmStopListAdd(UDM_STOPLIST *List,UDM_STOPWORD * stopword)
+
+int
+UdmStopListAdd(UDM_STOPLIST *List,UDM_STOPWORD * stopword)
 {
   size_t j;
   
@@ -91,9 +96,9 @@ int UdmStopListAdd(UDM_STOPLIST *List,UDM_STOPWORD * stopword)
     It will allow to avoid troubles with language guesser
   */
   
-  for(j= 0; j < List->nstopwords; j++)
+  for (j= 0; j < List->nstopwords; j++)
   {
-    if(!strcmp(List->StopWord[j].word,stopword->word))
+    if (!strcmp(List->StopWord[j].word, stopword->word))
     {
       UDM_FREE(List->StopWord[j].lang);
       List->StopWord[j].lang= (char*)UdmStrdup("");
@@ -109,20 +114,23 @@ int UdmStopListAdd(UDM_STOPLIST *List,UDM_STOPWORD * stopword)
   return 1;
 }
 
-void UdmStopListFree(UDM_STOPLIST *List)
+
+void
+UdmStopListFree(UDM_STOPLIST *List)
 {
   size_t i;
-  for(i=0; i < List->nstopwords; i++)
+  for(i= 0; i < List->nstopwords; i++)
   {
     UDM_FREE(List->StopWord[i].word);
     UDM_FREE(List->StopWord[i].lang);
   }
   UDM_FREE(List->StopWord);
-  List->nstopwords=0;
+  List->nstopwords= 0;
 }
 
 
-__C_LINK int __UDMCALL UdmStopListLoad(UDM_ENV * Conf,const char *fname)
+__C_LINK int __UDMCALL
+UdmStopListLoad(UDM_ENV *Conf,const char *fname)
 {
   char str[1024];
   char *lasttok, *lwrd, *charset=NULL;
@@ -144,7 +152,7 @@ __C_LINK int __UDMCALL UdmStopListLoad(UDM_ENV * Conf,const char *fname)
   
   bzero((void*)&stopword, sizeof(stopword));
   
-  while(fgets(str,sizeof(str),stopfile))
+  while (fgets(str,sizeof(str),stopfile))
   {
     if(!str[0])continue;
     if(str[0]=='#')continue;
