@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@
 #include "udm_doc.h"
 #include "udm_db.h"
 #include "udm_guesser.h"
-
+#include "udm_proto.h"
 
 int UdmCachedCopyUnpack(UDM_DOCUMENT *Doc, const char *sval, size_t l)
 {
@@ -83,6 +83,103 @@ int UdmCachedCopyUnpack(UDM_DOCUMENT *Doc, const char *sval, size_t l)
 }
 
 
+static int *
+UdmGetExcerptSourceFromContent(UDM_AGENT *Agent,
+                               UDM_RESULT *Res,
+                               UDM_DOCUMENT *Doc,
+                               UDM_CHARSET *bcs,
+                               char *Source, size_t SourceLen,
+                               size_t *length)
+{
+  size_t dstmaxlen;
+  char *dst;
+  UDM_HTMLTOK tag;
+  const char *htok, *last;
+  UDM_CHARSET *sys_int= &udm_charset_sys_int;
+  UDM_CHARSET *dcs= UdmGetCharSet(UdmVarListFindStr(&Doc->Sections, "Charset",
+                                  UdmVarListFindStr(&Doc->Sections, "Server-Charset", "iso-8859-1")));
+  int hlstop= UdmVarListFindBool(&Agent->Conf->Vars, "ExcerptStopword", 1);
+  const char *content_type= UdmVarListFindStr(&Doc->Sections, "Content-Type", NULL);
+  const char *deftype=  UdmVarListFindStr(&Agent->Conf->Vars,"DefaultContentType",NULL);
+  const char *seg= UdmVarListFindStr(&Agent->Conf->Vars, "Segmenter", NULL);
+  int segmenter= seg ? UdmUniSegmenterFind(Agent, NULL, seg) : 0;
+
+  if (!content_type ||
+      (strcmp(content_type, "text/plain") &&
+       strcmp(content_type, "text/xml") &&
+       strcmp(content_type, "text/vnd.wap.wml") &&
+       strcmp(content_type, "text/html")))
+    content_type= UdmGuessContentType(Source, SourceLen, deftype);
+
+  /* Make sure sizeof(int) alignment */
+  dstmaxlen= ((SourceLen * 14 + 10) / sizeof(int)) * sizeof(int);
+  dst= (char*) UdmMalloc(dstmaxlen);
+
+  if (!strcmp(content_type, "text/xml") ||
+      !strcmp(content_type, "text/vnd.wap.wml"))
+  {
+    UDM_DSTR buf;
+    UdmDSTRInit(&buf, 1024);
+    UdmHTMLTOKInit(&tag);
+    htok= UdmHTMLToken(Source, &last, &tag);
+    do
+    {
+      if (tag.type == UDM_HTML_TXT &&
+          !tag.script && !tag.comment && ! tag.style)
+        UdmDSTRAppend(&buf, htok, last - htok);
+      else
+        UdmDSTRAppend(&buf, " ", 1);
+    } while ((htok = UdmHTMLToken(NULL, &last, &tag)));
+    *length= UdmHlConvertExt(Agent, dst, dstmaxlen, &Res->WWList, bcs,
+                             buf.data, buf.size_data, dcs, sys_int,
+                             hlstop, segmenter);
+    *length /= sizeof(int);
+    UdmDSTRFree(&buf);
+  }
+  else if (!strcmp(content_type, "text/html"))
+  {
+    int i20= 0x20, i0= 0;
+    size_t dstlen= 0;
+    UDM_CONV lc_uni, uni_bc, uni_wcs;
+    UdmHTMLTOKInit(&tag);
+    UdmConvInit(&lc_uni, dcs, &udm_charset_sys_int, UDM_RECODE_HTML);
+    UdmConvInit(&uni_bc, &udm_charset_sys_int, &udm_charset_sys_int, UDM_RECODE_HTML);
+    UdmConvInit(&uni_wcs, &udm_charset_sys_int, bcs, UDM_RECODE_HTML);
+
+    for (htok= UdmHTMLToken(Source, &last, &tag);
+         htok;
+         htok = UdmHTMLToken(NULL, &last, &tag))
+    {
+      if (tag.type == UDM_HTML_TXT && tag.body &&
+          !tag.script && !tag.comment && ! tag.style)
+      {
+        UDM_ASSERT(dstmaxlen > dstlen);
+        dstlen+= UdmHlConvertExtWithConv(Agent,
+                                         dst + dstlen, dstmaxlen - dstlen,
+                                         &Res->WWList, htok, last - htok,
+                                         &uni_wcs, &lc_uni, &uni_bc,
+                                         hlstop, segmenter);
+      }
+      else
+      {
+        /* Put space character */
+        memcpy((void*)(dst+dstlen), (void*) &i20, sizeof(i20));
+        dstlen+= sizeof(i20);
+      }
+    } 
+    /* Put 0 terminator */
+    memcpy((void*)(dst+dstlen), (void*) &i0, sizeof(i0));
+    *length= dstlen / sizeof(int);
+  }
+  else
+  {
+    *length= UdmHlConvertExt(Agent, dst, dstmaxlen, &Res->WWList, bcs,
+                             Source, SourceLen, dcs, sys_int,
+                             hlstop, 0);
+    *length /= sizeof(int);
+  }
+  return (int*) dst;
+}
 
 
 static int
@@ -93,31 +190,18 @@ static int
                                size_t *length)
 {
 #ifdef HAVE_ZLIB
-  size_t l, dstmaxlen;
+  size_t l;
   char *Source, *in_buf, *dst;
   z_stream zstream;
-  UDM_CHARSET *sys_int= &udm_charset_sys_int;
   UDM_CHARSET *dcs= UdmGetCharSet(UdmVarListFindStr(&Doc->Sections, "Charset", "iso-8859-1"));
   const char *CachedCopy= UdmVarListFindStr(&Doc->Sections, "CachedCopy", NULL);
-  const char *content_type= UdmVarListFindStr(&Doc->Sections, "Content-Type", NULL);
-  const char *deftype=  UdmVarListFindStr(&Agent->Conf->Vars,"DefaultContentType",NULL);
-  UDM_HTMLTOK tag;
-  const char *htok, *last;
-  int hlstop= UdmVarListFindBool(&Agent->Conf->Vars, "ExcerptStopword", 1);
 #ifdef EXTRA_DEBUG
-  unsigned long long ticks;
+  unsigned long long ticks UdmStartTimer();
 #endif
   
-  if (!CachedCopy || !sys_int || !dcs)
+  if (!CachedCopy || !dcs || !(Source = UdmMalloc(UDM_MAXDOCSIZE)))
     return NULL;
 
-#ifdef EXTRA_DEBUG  
-  ticks= UdmStartTimer();
-#endif
-
-  if (!(Source = UdmMalloc(UDM_MAXDOCSIZE)))
-    return NULL;
-  
   l= strlen(CachedCopy);
   if (!(in_buf= UdmMalloc(l)))
   {
@@ -153,76 +237,11 @@ static int
   ticks= UdmStartTimer();
 #endif
 
-  if (!content_type ||
-      (strcmp(content_type, "text/plain") &&
-       strcmp(content_type, "text/xml") &&
-       strcmp(content_type, "text/vnd.wap.wml") &&
-       strcmp(content_type, "text/html")))
-    content_type= UdmGuessContentType(Source, zstream.total_out, deftype);
 
-  /* Make sure sizeof(int) alignment */
-  dstmaxlen= ((zstream.total_out * 14 + 10) / sizeof(int)) * sizeof(int);
-  dst= (char*) UdmMalloc(dstmaxlen);
 
-  if (!strcmp(content_type, "text/xml") ||
-      !strcmp(content_type, "text/vnd.wap.wml"))
-  {
-    UDM_DSTR buf;
-    UdmDSTRInit(&buf, 1024);
-    UdmHTMLTOKInit(&tag);
-    htok= UdmHTMLToken(Source, &last, &tag);
-    do
-    {
-      if (tag.type == UDM_HTML_TXT &&
-          !tag.script && !tag.comment && ! tag.style)
-        UdmDSTRAppend(&buf, htok, last - htok);
-      else
-        UdmDSTRAppend(&buf, " ", 1);
-    } while ((htok = UdmHTMLToken(NULL, &last, &tag)));
-    *length= UdmHlConvertExt(dst, dstmaxlen, &Res->WWList, bcs,
-                             buf.data, buf.size_data, dcs, sys_int, hlstop);
-    *length /= sizeof(int);
-    UdmDSTRFree(&buf);
-  }
-  else if (!strcmp(content_type, "text/html"))
-  {
-    int i20= 0x20, i0= 0;
-    size_t dstlen= 0;
-    UDM_CONV lc_uni, uni_bc, uni_wcs;
-    UdmHTMLTOKInit(&tag);
-    UdmConvInit(&lc_uni, dcs, &udm_charset_sys_int, UDM_RECODE_HTML);
-    UdmConvInit(&uni_bc, &udm_charset_sys_int, &udm_charset_sys_int, UDM_RECODE_HTML);
-    UdmConvInit(&uni_wcs, &udm_charset_sys_int, bcs, UDM_RECODE_HTML);
-
-    for (htok= UdmHTMLToken(Source, &last, &tag);
-         htok;
-         htok = UdmHTMLToken(NULL, &last, &tag))
-    {
-      if (tag.type == UDM_HTML_TXT && tag.body &&
-          !tag.script && !tag.comment && ! tag.style)
-      {
-        UDM_ASSERT(dstmaxlen > dstlen);
-        dstlen+= UdmHlConvertExtWithConv(dst + dstlen, dstmaxlen - dstlen,
-                                         &Res->WWList, htok, last - htok,
-                                         &uni_wcs, &lc_uni, &uni_bc, hlstop);
-      }
-      else
-      {
-        /* Put space character */
-        memcpy((void*)(dst+dstlen), (void*) &i20, sizeof(i20));
-        dstlen+= sizeof(i20);
-      }
-    } 
-    /* Put 0 terminator */
-    memcpy((void*)(dst+dstlen), (void*) &i0, sizeof(i0));
-    *length= dstlen / sizeof(int);
-  }
-  else
-  {
-    *length= UdmHlConvertExt(dst, dstmaxlen, &Res->WWList, bcs,
-                             Source, zstream.total_out, dcs, sys_int, hlstop);
-    *length /= sizeof(int);
-  }
+  dst= (char*) UdmGetExcerptSourceFromContent(Agent, Res, Doc, bcs,
+                                              Source, zstream.total_out,
+                                              length);
   
   UdmFree(Source);
 
@@ -251,7 +270,7 @@ static int
   UDM_CHARSET *sys_int= &udm_charset_sys_int;
 
 #ifndef NO_ADVANCE_CONV
-  size_t  ul;
+  size_t  ul, charlen;
   UDM_CONV conv;
   
   if (!Source || !sys_int || !bcs)
@@ -267,8 +286,9 @@ static int
     UdmFree(res);
     return(NULL);
   }
-  res[ul / sizeof(int)]= 0;
-  *length= ul / sizeof(int);
+  charlen= ul / sizeof(int);
+  res[charlen]= 0;
+  *length= charlen;
 #else
   {
     int hlstop= UdmVarListFindBool(&Agent->Conf->Vars, "ExcerptStopword", 1);
@@ -328,7 +348,7 @@ UdmExcerptDoc (UDM_AGENT *Agent,
   UDM_CHARSET *sys_int= &udm_charset_sys_int;
   UDM_CHARSET *bcs= Agent->Conf->bcs;
   UDM_DSTR buf;
-  int dots[]= { 0x2E, 0x2E, 0x2E };
+  int dots[]= { 0x2E, 0x2E, 0x2E }; /* "..." */
 #ifdef EXTRA_DEBUG
   unsigned long ticks;
 #endif
@@ -340,7 +360,30 @@ UdmExcerptDoc (UDM_AGENT *Agent,
   ticks= UdmStartTimer();
 #endif
 
-  if (!(Source= UdmGetExcerptSourceCachedCopy(Agent, Res, Doc, bcs, &ul)) &&
+  if (UdmVarListFindBool(&Agent->Conf->Vars, "UseLocalCachedCopy", 0))
+  {
+    const char *url= UdmVarListFindStr(&Doc->Sections, "url", NULL);
+    int rc= url ? UdmGetURLSimple(Agent, Doc, url) : UDM_ERROR;
+    size_t csize= Doc->Buf.size - (Doc->Buf.content-Doc->Buf.buf);
+    
+    if (0)
+    {
+      const char *cl= UdmVarListFindStr(&Doc->Sections, "Content-Length", "?");
+      const char *ct= UdmVarListFindStr(&Doc->Sections, "Content-Type", "?");
+      const char *ce= UdmVarListFindStr(&Doc->Sections, "Content-Encoding", "?");
+      fprintf(stderr, "rc=%d url=%s size=%d cl=%s ce=%s ct=%s b.ct=%p b.cl=%d csize=%d\n",
+              rc, url, Doc->Buf.size, cl, ce, ct, Doc->Buf.content,
+              Doc->Buf.content_length, csize);
+    }
+    
+    Source= UdmGetExcerptSourceFromContent(Agent, Res, Doc, bcs,
+                                           Doc->Buf.content, csize, &ul);
+  }
+  else
+    Source= NULL;
+
+  if (!Source &&
+      !(Source= UdmGetExcerptSourceCachedCopy(Agent, Res, Doc, bcs, &ul)) &&
       !(Source= UdmGetExcerptSourceBody(Agent, Res, Doc, bcs, &ul)))
     return(NULL);
 
@@ -377,32 +420,79 @@ UdmExcerptDoc (UDM_AGENT *Agent,
   UdmDSTRInit(&buf, 1024);
   for (i= 0; i < ul; i++)
   {
-    if (Source[i] == 2)
+    if (Source[i] == 2) /* Beginning of a highlighted area found */
     {
-      for (j= i + 1; j < ul; j++)
+      for (j= i + 1; j < ul; j++) /* Find the end of highighted area */
         if (Source[j] == 3)
           break;
       
+      /*
+        Move towards the beginning of the document
+        and find where to break excerpt:
+        - either on the closest space character
+        - or put all "ExcerptPadding" characters if no space found.
+          This is very likely in case of Asian texts with no spaces.
+      */
       left= ExcerptPadding < i ? i - ExcerptPadding : 0;
       if (left < prev_right)
+      {
         left= prev_right;
-      if (left) 
-        while (left < i && Source[left] != 0x20)
-          left++;
+      }
+      if (left)
+      {
+        size_t left0= left;
+        for ( ; Source[left] != 0x20; left++)
+        {
+          if (left >= i) /* no space found */
+          {
+            left= left0;
+            break;
+          }
+        }
+      }
       
+      /*
+        Now move towards the end of the document and
+        find the right excerpt border:
+        - either on the closest space character
+        - or after ExcerptSize characters
+      */
       right= ExcerptPadding + j;
       if (right >= ul)
+      {
         right= ul - 1;
+      }
       else
-        while (right > j && Source[right] != 0x20)
-          right--;
+      {
+        size_t right0= right;
+        for ( ; Source[right] != 0x20 ;  right--)
+        {
+          if (right <= j) /* No space found */
+          {
+            right= right0;
+            break;
+          }
+        }
+      }
       
+      /*
+        Check if there is a space for the new excerpt.
+        We must stay under ExcerptSize limit.
+      */
       if (ExcerptSize < buf.size_data / sizeof(int) + right - left + 1)
         break;
       if (left != prev_right)
         UdmDSTRAppend(&buf, (char *)dots, sizeof(dots));
       UdmDSTRAppend(&buf, (char *)&Source[left], (right - left + 1) * sizeof(int));
       i= right;
+
+       /*
+         On the next step we should not add dots if the new highlighted word
+         immediately follows this word: <02>W1<03><02><W2>
+         This is possible in case of "Segmenter cjk".
+       */
+      if (Source[right] == 0x03)
+        right++;
       prev_right= right;
     }
   }
@@ -420,6 +510,7 @@ UdmExcerptDoc (UDM_AGENT *Agent,
     return NULL;
   }
 
+  /* Convert Unicode to BrowserCharset */
   UdmConvInit(&conv, sys_int, bcs, UDM_RECODE_HTML);
   l= UdmConv(&conv, res, ul, buf.data, buf.size_data);
   UdmDSTRFree(&buf);
