@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2009 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -111,6 +111,8 @@ static char *UdmConvDup(const char *src,
 }
 
 
+#define UDM_TEMPLATE_BASE64 0xB64
+
 size_t UdmTemplatePrintVar(UDM_ENV *Env, 
                            FILE *stream, char *dst, size_t dstlen,
                            const char *value, int type,
@@ -151,6 +153,13 @@ size_t UdmTemplatePrintVar(UDM_ENV *Env,
       UdmEscapeURL(eval,value);
       res= out_string(stream, dst, dstlen, eval);
       break;
+    case UDM_TEMPLATE_BASE64:
+      {
+        size_t len= strlen(value);
+        eval= (char*)UdmMalloc(BASE64_LEN(len));
+        udm_base64_encode(value, eval, len);
+        res= out_string(stream, dst, dstlen, eval);
+      }
   }
   UDM_FREE(eval);
   return res;
@@ -236,6 +245,11 @@ static size_t PrintTextTemplate(UDM_AGENT *A, FILE *stream, char *dst,
         vbeg=s+3;
         type='^';
       }
+      else if (!strncasecmp(s, "$base64(", 8))
+      {
+        vbeg= s + 8;
+        type= UDM_TEMPLATE_BASE64;
+      }
       
       for (vend=s; vend[0]; vend++)
       {
@@ -706,52 +720,24 @@ static int TemplateInclude(UDM_TEMPLATE *T)
   UDM_TMPL_PRG *p= &T->prg;  
   UDM_TMPL_PRGITEM *it= &p->Items[p->curr];
   const char *tag_content= it->arg[1];
-  size_t max_doc_size= (size_t)UdmVarListFindInt(vars,"MaxDocSize",UDM_MAXDOCSIZE);
-  UDM_DOCUMENT Inc;
 
   if (!Agent)
     return UDM_OK;
   
-  UdmDocInit(&Inc);
-  if(!Inc.Buf.buf) Inc.Buf.buf = (char*)UdmMalloc(max_doc_size);
-  Inc.Buf.maxsize=max_doc_size;
 
   if(it->arg[1])
   {
-    const char *ce;
+    UDM_DOCUMENT Inc;
     size_t vurlen = 256 + 4 * strlen(tag_content);
     char  *vurl = (char*)UdmMalloc(vurlen);
-          
     PrintTextTemplate(Agent, NULL, vurl, vurlen, vars, it->arg[1], T->HlBeg, T->HlEnd);
-    UdmURLParse(&Inc.CurURL,vurl);
-    UDM_FREE(vurl);
-    UdmVarListReplaceStr(&Inc.RequestHeaders, "Host", UDM_NULL2EMPTY(Inc.CurURL.hostname));
-    Inc.connp.hostname = (char*)UdmStrdup(UDM_NULL2EMPTY(Inc.CurURL.hostname));
-    Inc.connp.port = Inc.CurURL.port ? Inc.CurURL.port : Inc.CurURL.default_port;
-    
-    if(UdmHostLookup(&Agent->Conf->Hosts, &Inc.connp))
+
+    UdmDocInit(&Inc);
+        
+    if (UdmGetURLSimple(Agent, &Inc, vurl) == UDM_OK)
     {
-    }
-          
-    if(UdmGetURL(Agent,&Inc)==UDM_OK)
-    {
-      UdmParseHTTPResponse(Agent,&Inc);
-      if(Inc.Buf.content)
+      if (Inc.Buf.content)
       {
-        ce=UdmVarListFindStr(&Inc.Sections,"Content-Encoding","");
-#ifdef HAVE_ZLIB
-        if(!strcasecmp(ce,"gzip") || !strcasecmp(ce,"x-gzip")){
-          UdmUnGzip(&Inc);
-        }
-        else if(!strcasecmp(ce,"deflate"))
-        {
-          UdmInflate(&Inc);
-        }
-        else if(!strcasecmp(ce,"compress") || !strcasecmp(ce,"x-compress"))
-        {
-          UdmUncompress(&Inc);
-        }
-#endif
         if(T->stream)
         {
           fprintf(T->stream,"%s",Inc.Buf.content);
@@ -762,8 +748,9 @@ static int TemplateInclude(UDM_TEMPLATE *T)
         }
       }
     }
+    UdmDocFree(&Inc);
+    UDM_FREE(vurl);
   }
-  UdmDocFree(&Inc);
   return UDM_OK;
 }
 
@@ -1704,9 +1691,13 @@ int UdmTemplateLoad(UDM_AGENT *Agent,
 #ifdef HAVE_SQL
   if(Env->dbl.nitems == 0)
   {
-    if (UDM_OK!=UdmDBListAdd(&Env->dbl, dbaddr = "mysql://localhost/mnogosearch", UDM_OPEN_MODE_READ))
+    if (UDM_OK != (rc= UdmDBListAdd(&Env->dbl,
+                                    dbaddr= "mysql://localhost/mnogosearch",
+                                    UDM_OPEN_MODE_READ)))
     {
-      sprintf(Env->errstr,"Invalid DBAddr: '%s'", (!dbaddr)?"NULL":dbaddr);
+      sprintf(Env->errstr,"%s DBAddr: '%s'",
+              rc == UDM_UNSUPPORTED ? "Unsupported" : "Invalid",
+             (!dbaddr)?"NULL":dbaddr);
       rc= UDM_ERROR;
       goto ret;
     }
@@ -1714,9 +1705,9 @@ int UdmTemplateLoad(UDM_AGENT *Agent,
 #endif
   if(Env->dbl.nitems == 0)
   {
-    if (UDM_OK!=UdmDBListAdd(&Env->dbl, dbaddr =  "searchd://localhost/", UDM_OPEN_MODE_READ))
+    if (UDM_OK!= (rc= UdmDBListAdd(&Env->dbl, dbaddr=  "searchd://localhost/", UDM_OPEN_MODE_READ)))
     {
-      sprintf(Env->errstr,"Invalid DBAddr: '%s'", (!dbaddr)?"NULL":dbaddr);
+      sprintf(Env->errstr,"Invalid DBAddr: '%s'", (!dbaddr) ? "NULL" : dbaddr);
       rc= UDM_ERROR;
       goto ret;
     }
